@@ -17,43 +17,43 @@
  *
  */
 
+#include "body-pane.h"
+
 #include <config.h>
 #include <cctype>
 #include <cmath>
 #include <iostream>
 #include <sstream>
-extern "C" {
-  #include <glib/gi18n.h>
-  #include "gtk-compat.h"
-  #include <gmime/gmime.h>
-}
+#include <glib/gi18n.h>
+#include <gtk/gtk.h>
+#include <gdk/gdk.h>
+#include <gmime/gmime.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <pan/general/debug.h>
 #include <pan/general/log.h>
 #include <pan/general/macros.h>
 #include <pan/general/utf8-utils.h>
 #include <pan/general/e-util.h>
+#include "pan/gui/load-icon.h"
 #include <pan/usenet-utils/gnksa.h>
 #include <pan/usenet-utils/mime-utils.h>
 #include <pan/usenet-utils/url-find.h>
-#include <pan/icons/pan-pixbufs.h>
-#include "body-pane.h"
 #include "pad.h"
 #include "tango-colors.h"
 #include "xface.h"
 #include "url.h"
-#include "gtk-compat.h"
 #include "save-attach-ui.h"
 
 #ifdef HAVE_WEBKIT
   #include <webkit/webkit.h>
 #endif
 
+
 #define FIRST_PICTURE "first-picture"
 
 #define FACE_SIZE 48
 
-using namespace pan;
+namespace pan {
 
 /***
 ****
@@ -75,11 +75,11 @@ namespace
   };
 
   struct Icon {
-  const guint8 * pixbuf_txt;
+  const char * pixbuf_file;
   GdkPixbuf * pixbuf;
   } icons[NUM_ICONS] = {
-    { icon_sig_ok,          0 },
-    { icon_sig_fail,        0 }
+    { "icon_sig_ok.png",          nullptr },
+    { "icon_sig_fail.png",        nullptr }
   };
 }
 
@@ -172,7 +172,7 @@ namespace
     g_object_set_data (G_OBJECT(o), FULLSIZE, GINT_TO_POINTER(b));
   }
   bool get_fullsize_flag (gpointer o) {
-    return g_object_get_data (G_OBJECT(o), FULLSIZE) != 0;
+    return g_object_get_data (G_OBJECT(o), FULLSIZE) != nullptr;
   }
   bool toggle_fullsize_flag (gpointer o) {
     const bool b (!get_fullsize_flag (o));
@@ -194,11 +194,11 @@ namespace
     CURSOR_QTY
   };
 
-  GdkCursor ** cursors(0);
-  GdkCursor * cursor_current(0);
+  GdkCursor ** cursors(nullptr);
+  GdkCursor * cursor_current(nullptr);
 
   void free_cursors (void) {
-    delete cursors;
+    delete[] cursors;
   }
 
   void create_cursors() {
@@ -211,33 +211,15 @@ namespace
 
       int width, height;
       const GtkIconSize size (GTK_ICON_SIZE_LARGE_TOOLBAR);
-#if !GTK_CHECK_VERSION(3,0,0)
-      GtkStyle * style (gtk_widget_get_style (w));
-      const GtkTextDirection dir (GTK_TEXT_DIR_NONE);
-      const GtkStateType state (GTK_STATE_PRELIGHT);
-
-      GtkIconSet * icon_set = gtk_style_lookup_icon_set (style, GTK_STOCK_ZOOM_IN);
-      GdkPixbuf * pixbuf = gtk_icon_set_render_icon (icon_set, style, dir, state, size, w, NULL);
-      g_object_get (G_OBJECT(pixbuf), "width", &width, "height", &height, NULL);
-      cursors[CURSOR_ZOOM_IN] = gdk_cursor_new_from_pixbuf (display, pixbuf, width/2, height/2);
-      g_object_unref (G_OBJECT(pixbuf));
-
-      icon_set = gtk_style_lookup_icon_set (style, GTK_STOCK_ZOOM_OUT);
-      pixbuf = gtk_icon_set_render_icon (icon_set, style, dir, state, size, w, NULL);
-      g_object_get (G_OBJECT(pixbuf), "width", &width, "height", &height, NULL);
-      cursors[CURSOR_ZOOM_OUT] = gdk_cursor_new_from_pixbuf (display, pixbuf, width/2, height/2);
-      g_object_unref (G_OBJECT(pixbuf));
-#else
       GdkPixbuf * pixbuf = gtk_widget_render_icon_pixbuf (w, GTK_STOCK_ZOOM_IN, size);
-      g_object_get (G_OBJECT(pixbuf), "width", &width, "height", &height, NULL);
+      g_object_get (G_OBJECT(pixbuf), "width", &width, "height", &height, nullptr);
       cursors[CURSOR_ZOOM_IN] = gdk_cursor_new_from_pixbuf (display, pixbuf, width/2, height/2);
       g_object_unref (G_OBJECT(pixbuf));
 
       pixbuf = gtk_widget_render_icon_pixbuf (w, GTK_STOCK_ZOOM_OUT, size);
-      g_object_get (G_OBJECT(pixbuf), "width", &width, "height", &height, NULL);
+      g_object_get (G_OBJECT(pixbuf), "width", &width, "height", &height, nullptr);
       cursors[CURSOR_ZOOM_OUT] = gdk_cursor_new_from_pixbuf (display, pixbuf, width/2, height/2);
       g_object_unref (G_OBJECT(pixbuf));
-#endif
 
       cursors[CURSOR_IBEAM] = gdk_cursor_new (GDK_XTERM);
       cursors[CURSOR_HREF] = gdk_cursor_new (GDK_HAND2);
@@ -249,7 +231,7 @@ namespace
     GdkCursor * cursor_new = cursors[mode];
     if (cursor_new != cursor_current) {
        gdk_window_set_cursor (window, cursor_current=cursor_new);
-       cursor_unref (cursor_current);
+       g_object_unref(cursor_current);
     }
   }
 
@@ -303,8 +285,8 @@ namespace
     {
       GtkTextIter begin(*iter), end(*iter);
       if (!gtk_text_iter_begins_tag (&begin, url_tag))
-        gtk_text_iter_backward_to_tag_toggle (&begin, NULL);
-      gtk_text_iter_forward_to_tag_toggle (&end, NULL);
+        gtk_text_iter_backward_to_tag_toggle (&begin, nullptr);
+      gtk_text_iter_forward_to_tag_toggle (&end, nullptr);
       char * pch = gtk_text_iter_get_text (&begin, &end);
       if (pch) {
         ret = pch;
@@ -318,11 +300,11 @@ namespace
                                 GdkEventMotion  * event,
                                 gpointer          hover_url)
   {
-    if (event->window != NULL)
+    if (event->window != nullptr)
     {
       int x, y;
       if (event->is_hint)
-        gdk_window_get_device_position (event->window, event->device, &x, &y, NULL);
+        gdk_window_get_device_position (event->window, event->device, &x, &y, nullptr);
       else {
         x = (int) event->x;
         y = (int) event->y;
@@ -344,7 +326,7 @@ namespace
     const int nw (size ? size->width : 0);
     const int nh (size ? size->height : 0);
 
-    GdkPixbuf * out (0);
+    GdkPixbuf * out (nullptr);
     if (nw>=100 && nh>=100)
     {
       const int ow (gdk_pixbuf_get_width (pixbuf));
@@ -376,12 +358,12 @@ namespace
 
     const int begin_offset (gtk_text_iter_get_offset (iter));
 
-    GdkPixbuf * original (0);
-    GdkPixbuf * old_scaled (0);
+    GdkPixbuf * original (nullptr);
+    GdkPixbuf * old_scaled (nullptr);
     if (!get_pixbuf_at_offset (buf, begin_offset, original, old_scaled))
       return;
 
-    GdkPixbuf * new_scaled (size_to_fit (original, (fullsize ? 0 : size)));
+    GdkPixbuf * new_scaled (size_to_fit (original, (fullsize ? nullptr : size)));
     const int old_w (gdk_pixbuf_get_width (old_scaled));
     const int new_w (gdk_pixbuf_get_width (new_scaled));
     const int old_h (gdk_pixbuf_get_height (old_scaled));
@@ -425,7 +407,7 @@ BodyPane :: mouse_button_pressed (GtkWidget *w, GdkEventButton *event)
       /* this is a crude way of making sure that double-click
        * doesn't open two or three browser windows. */
       static time_t last_url_time (0);
-      const time_t this_url_time (time (0));
+      const time_t this_url_time (time (nullptr));
       if (this_url_time != last_url_time) {
         last_url_time = this_url_time;
         URL :: open (_prefs, url.c_str());
@@ -436,9 +418,9 @@ BodyPane :: mouse_button_pressed (GtkWidget *w, GdkEventButton *event)
       get_iter_from_event_coords (w, (int)event->x, (int)event->y, &iter);
       if (gtk_text_iter_has_tag (&iter, pix_tag))
       {
-        if (!gtk_text_iter_begins_tag (&iter, pix_tag))
+        if (!gtk_text_iter_starts_tag (&iter, pix_tag))
           gtk_text_iter_backward_to_tag_toggle (&iter, pix_tag);
-        g_assert (gtk_text_iter_begins_tag (&iter, pix_tag));
+        g_assert (gtk_text_iter_starts_tag (&iter, pix_tag));
 
         // percent_x,percent_y reflect where the user clicked in the picture
         GdkRectangle rec;
@@ -461,7 +443,7 @@ BodyPane :: mouse_button_pressed (GtkWidget *w, GdkEventButton *event)
         set_cursor_from_iter (event->window, w, &iter);
 
         // x2,y2 are to position percent_x,percent_y in the middle of the window.
-        GtkTextMark * mark = gtk_text_buffer_create_mark (buf, NULL, &iter, true);
+        GtkTextMark * mark = gtk_text_buffer_create_mark (buf, nullptr, &iter, true);
         const double x2 = CLAMP ((percent_x + (percent_x - 0.5)), 0.0, 1.0);
         const double y2 = CLAMP ((percent_y + (percent_y - 0.5)), 0.0, 1.0);
         gtk_text_view_scroll_to_mark (GTK_TEXT_VIEW(w), mark, 0.0, true, x2, y2);
@@ -507,37 +489,37 @@ namespace
     g_object_set (get_or_create_tag (table, "text"),
       "foreground", p.get_color_str ("text-color-fg", fg).c_str(),
       "background", p.get_color_str ("text-color-bg", bg).c_str(),
-      NULL);
+      nullptr);
     g_object_set (get_or_create_tag(table,"bold"),
       "weight", PANGO_WEIGHT_BOLD,
-      NULL);
+      nullptr);
     g_object_set (get_or_create_tag (table, "italic"),
       "style", PANGO_STYLE_ITALIC,
-      NULL);
+      nullptr);
     g_object_set (get_or_create_tag (table, "underline"),
       "underline", PANGO_UNDERLINE_SINGLE,
-      NULL);
+      nullptr);
     g_object_set (get_or_create_tag (table, "url"),
       "underline", PANGO_UNDERLINE_SINGLE,
       "foreground", p.get_color_str ("body-pane-color-url", TANGO_SKY_BLUE_DARK).c_str(),
       "background", p.get_color_str ("body-pane-color-url-bg", bg).c_str(),
-      NULL);
+      nullptr);
     g_object_set (get_or_create_tag (table, "quote_1"),
       "foreground", p.get_color_str ("body-pane-color-quote-1", TANGO_CHAMELEON_DARK).c_str(),
       "background", p.get_color_str ("body-pane-color-quote-1-bg", bg).c_str(),
-      NULL);
+      nullptr);
     g_object_set (get_or_create_tag (table, "quote_2"),
       "foreground", p.get_color_str ("body-pane-color-quote-2", TANGO_ORANGE_DARK).c_str(),
       "background", p.get_color_str ("body-pane-color-quote-2-bg", bg).c_str(),
-      NULL);
+      nullptr);
     g_object_set (get_or_create_tag (table, "quote_3"),
       "foreground", p.get_color_str ("body-pane-color-quote-3", TANGO_PLUM_DARK).c_str(),
       "background", p.get_color_str ("body-pane-color-quote-3-bg", bg).c_str(),
-      NULL);
+      nullptr);
     g_object_set (get_or_create_tag (table, "signature"),
       "foreground", p.get_color_str ("body-pane-color-signature", TANGO_SKY_BLUE_LIGHT).c_str(),
       "background", p.get_color_str ("body-pane-color-signature-bg", bg).c_str(),
-      NULL);
+      nullptr);
   }
 }
 
@@ -592,18 +574,18 @@ namespace
    *
    * @param utf8_line the line whose quote status we're checking
    * @param utf8_byte_len the byte length of utf8_line
-   * @return a const string for the line's quote tag.  Never NULL.
+   * @return a const string for the line's quote tag.  Never nullptr.
    */
   const char * get_quote_tag (const TextMassager     * text_massager,
                               const char             * utf8_line,
                               int                      utf8_byte_len)
   {
     const char * str = utf8_line;
-    const char * line_end = utf8_line + utf8_byte_len;
     const char * retval = "quote_0";
 
     if (0<utf8_byte_len && str && *str)
     {
+      const char * line_end = utf8_line + utf8_byte_len;
       int depth = 0;
 
       // walk past leading spaces
@@ -621,12 +603,13 @@ namespace
         str = g_utf8_next_char (str);
       }
 
-      if (!depth)
-        retval = "quote_0";
-      else switch (depth % 3) {
-        case 1: retval = "quote_1"; break;
-        case 2: retval = "quote_2"; break;
-        case 0: retval = "quote_3"; break;
+      if (depth != 0)
+      {
+        switch (depth % 3) {
+          case 1: retval = "quote_1"; break;
+          case 2: retval = "quote_2"; break;
+          case 0: retval = "quote_3"; break;
+        }
       }
     }
 
@@ -651,12 +634,12 @@ namespace
 
  void create_emoticons()
   {
-    emoticon_pixbufs[":)"] = gdk_pixbuf_new_from_inline (-1, icon_mozilla_smile, false, 0);
-    emoticon_pixbufs[":-)"] = gdk_pixbuf_new_from_inline (-1, icon_mozilla_smile, false, 0);
-    emoticon_pixbufs[";)"] = gdk_pixbuf_new_from_inline (-1, icon_mozilla_wink, false, 0);
-    emoticon_pixbufs[":("] = gdk_pixbuf_new_from_inline (-1, icon_mozilla_frown, false, 0);
-    emoticon_pixbufs[":P"] = gdk_pixbuf_new_from_inline (-1, icon_mozilla_tongueout, false, 0);
-    emoticon_pixbufs[":O"] = gdk_pixbuf_new_from_inline (-1, icon_mozilla_surprised, false, 0);
+    emoticon_pixbufs[":)"] = load_icon("icon_mozilla_smile.png");
+    emoticon_pixbufs[":-)"] = load_icon("icon_mozilla_smile.png");
+    emoticon_pixbufs[";)"] = load_icon("icon_mozilla_wink.png");
+    emoticon_pixbufs[":("] = load_icon("icon_mozilla_frown.png");
+    emoticon_pixbufs[":P"] = load_icon("icon_mozilla_tongueout.png");
+    emoticon_pixbufs[":O"] = load_icon("icon_mozilla_surprised.png");
   }
 
   enum TagMode { ADD, REPLACE };
@@ -698,7 +681,7 @@ namespace
     {
       GtkTextTag * sig_tag (gtk_text_tag_table_lookup (tags, "signature"));
       if (sig_tag)
-        g_object_set (sig_tag, "invisible", !show, NULL);
+        g_object_set (sig_tag, "invisible", !show, nullptr);
     }
   }
 
@@ -710,7 +693,7 @@ namespace
                                      GdkPixbuf          * pixbuf)
   {
     g_assert (!text.empty());
-    g_assert (pixbuf != 0);
+    g_assert (pixbuf != nullptr);
 
     GtkTextTagTable * tags (gtk_text_buffer_get_tag_table (buffer));
     GtkTextTag * url_tag (gtk_text_tag_table_lookup (tags, "url"));
@@ -752,7 +735,7 @@ namespace
                              bool                  do_urls)
   {
 
-    g_return_if_fail (buffer!=0);
+    g_return_if_fail (buffer!=nullptr);
     g_return_if_fail (GTK_IS_TEXT_BUFFER(buffer));
 
     // mute the quoted text, if desired
@@ -777,7 +760,7 @@ namespace
     gtk_text_buffer_apply_tag_by_name (buffer, "text", &mark_start, &end);
 
     // find where the signature begins...
-    const char * sig_point (0);
+    const char * sig_point (nullptr);
     int offset (0);
     if (GNKSA::find_signature_delimiter (v, offset) != GNKSA::SIG_NONE)
       sig_point = v.str + offset;
@@ -859,7 +842,7 @@ namespace
 
           if (e) {
             already_processed.insert (e);
-            const char * type (0);
+            const char * type (nullptr);
             switch (*b) {
               case '*': type = "bold"; break;
               case '_': type = "underline"; break;
@@ -878,8 +861,11 @@ namespace
       StringView area;
       StringView march (v_all);
       while ((url_find (march, area))) {
+        // Do no highlight URL in signature when not showing signature
+        if (area.str > sig_point && !show_sig)
+          break;
         set_section_tag (buffer, &start, v_all, area, "url", REPLACE);
-        march = march.substr (area.str + area.len, 0);
+        march = march.substr (area.str + area.len, nullptr);
       }
     }
 
@@ -902,14 +888,10 @@ namespace
   GdkPixbuf* get_pixbuf_from_gmime_part (GMimePart * part)
   {
     GdkPixbufLoader * l (gdk_pixbuf_loader_new ());
-    GError * err (0);
+    GError * err (nullptr);
 
     // populate the loader
-#ifdef HAVE_GMIME_30
     GMimeDataWrapper * wrapper (g_mime_part_get_content (part));
-#else    
-    GMimeDataWrapper * wrapper (g_mime_part_get_content_object (part));
-#endif    
     if (wrapper)
     {
       GMimeStream * mem_stream (g_mime_stream_mem_new ());
@@ -947,7 +929,7 @@ namespace
     }
 
     // create the pixbuf
-    GdkPixbuf * pixbuf (0);
+    GdkPixbuf * pixbuf (nullptr);
     if (!err)
       pixbuf = gdk_pixbuf_loader_get_pixbuf (l);
     else {
@@ -989,9 +971,9 @@ BodyPane :: append_part (GMimeObject * parent, GMimeObject * obj, GtkAllocation
   {
     GdkPixbuf * original (get_pixbuf_from_gmime_part (part));
     const bool fullsize (!_prefs.get_flag ("size-pictures-to-fit", true));
-    GdkPixbuf * scaled (size_to_fit (original, fullsize ? 0 : widget_size));
+    GdkPixbuf * scaled (size_to_fit (original, fullsize ? nullptr : widget_size));
 
-    if (scaled != 0)
+    if (scaled != nullptr)
     {
       GtkTextIter iter;
 
@@ -1001,7 +983,7 @@ BodyPane :: append_part (GMimeObject * parent, GMimeObject * obj, GtkAllocation
         gtk_text_buffer_insert (_buffer, &iter, "\n", -1);
 
       // rembember the location of the first picture.
-      if (gtk_text_buffer_get_mark (_buffer, FIRST_PICTURE) == NULL)
+      if (gtk_text_buffer_get_mark (_buffer, FIRST_PICTURE) == nullptr)
         gtk_text_buffer_create_mark (_buffer, FIRST_PICTURE, &iter, true);
         gtk_text_buffer_create_mark (_buffer, FIRST_PICTURE, &iter, true);
 
@@ -1084,7 +1066,7 @@ namespace
   {
     const char * val (message ? g_mime_object_get_header ((GMimeObject *)message, key) : "");
     const std::string utf8_val (header_to_utf8 (val, fallback_charset));
-    char * e (0);
+    char * e (nullptr);
     if (strcmp (key, "From"))
       e = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>", utf8_val.c_str());
     else {
@@ -1129,7 +1111,7 @@ namespace
 void
 BodyPane :: set_text_from_message (GMimeMessage * message)
 {
-  const char * fallback_charset (_charset.empty() ? 0 : _charset.c_str());
+  const char * fallback_charset (_charset.empty() ? nullptr : _charset.c_str());
 
   // mandatory headers...
   std::string s;
@@ -1192,18 +1174,14 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
   s.resize (std::max((size_t)0,s.size()-1)); // remove trailing linefeed
   gtk_label_set_markup (GTK_LABEL(_headers), s.c_str());
 
-#if !GTK_CHECK_VERSION(3,0,0)
-  // ellipsize mode is useless w/o this in expander...
-  gtk_label_set_width_chars (GTK_LABEL(_headers), (int)w);
-#endif
 
   // set the x-face...
   gtk_image_clear(GTK_IMAGE(_xface));
-  const char * pch = message ? g_mime_object_get_header ((GMimeObject *) message, "X-Face") : 0;
+  const char * pch = message ? g_mime_object_get_header ((GMimeObject *) message, "X-Face") : nullptr;
   if (pch && gtk_widget_get_window(_xface) )
   {
     gtk_widget_set_size_request (_xface, FACE_SIZE, FACE_SIZE);
-    GdkPixbuf *pixbuf = NULL;
+    GdkPixbuf *pixbuf = nullptr;
     pixbuf = pan_gdk_pixbuf_create_from_x_face (pch);
     gtk_image_set_from_pixbuf (GTK_IMAGE(_xface), pixbuf);
     g_object_unref (pixbuf);
@@ -1213,7 +1191,7 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
 
   // set the face
   gtk_image_clear(GTK_IMAGE(_face));
-  pch = message ? g_mime_object_get_header ((GMimeObject *) message, "Face") : 0;
+  pch = message ? g_mime_object_get_header ((GMimeObject *) message, "Face") : nullptr;
   if (pch && gtk_widget_get_window(_face))
   {
     gtk_widget_set_size_request (_face, FACE_SIZE, FACE_SIZE);
@@ -1221,9 +1199,9 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
     g_mime_encoding_init_decode(&dec, GMIME_CONTENT_ENCODING_BASE64);
     guchar* buf = new guchar[strlen(pch)];
     int len = g_mime_encoding_step(&dec, pch, strlen(pch), (char*)buf);
-    GdkPixbufLoader *pl = gdk_pixbuf_loader_new_with_type( "png", NULL);
-    gdk_pixbuf_loader_write(pl, buf, len, NULL);
-    gdk_pixbuf_loader_close(pl, NULL);
+    GdkPixbufLoader *pl = gdk_pixbuf_loader_new_with_type( "png", nullptr);
+    gdk_pixbuf_loader_write(pl, buf, len, nullptr);
+    gdk_pixbuf_loader_close(pl, nullptr);
     GdkPixbuf *pixbuf = gdk_pixbuf_loader_get_pixbuf(pl);
     gtk_image_set_from_pixbuf (GTK_IMAGE(_face), pixbuf);
     g_object_unref(pl);
@@ -1247,18 +1225,14 @@ BodyPane :: set_text_from_message (GMimeMessage * message)
   GtkTextIter start, end;
   gtk_text_buffer_get_bounds (_buffer, &start, &end);
   gtk_text_buffer_delete (_buffer, &start, &end);
-  if (gtk_text_buffer_get_mark (_buffer, FIRST_PICTURE) != NULL)
+  if (gtk_text_buffer_get_mark (_buffer, FIRST_PICTURE) != nullptr)
     gtk_text_buffer_delete_mark_by_name (_buffer, FIRST_PICTURE);
   clear_pixbuf_cache (_buffer);
 
   // maybe add the headers
   const bool do_show_headers (_prefs.get_flag ("show-all-headers", false));
   if (message && do_show_headers) {
-#ifdef HAVE_GMIME_30
-    char * headers (g_mime_object_get_headers ((GMimeObject *) message, NULL));
-#else    
-    char * headers (g_mime_object_get_headers ((GMimeObject *) message));
-#endif    
+    char * headers (g_mime_object_get_headers ((GMimeObject *) message, nullptr));
     GtkTextIter end;
     gtk_text_buffer_get_end_iter (_buffer, &end);
     StringView line, v(headers);
@@ -1331,7 +1305,7 @@ BodyPane :: on_verbose_tooltip_cb(GtkWidget  *widget,
   std::pair<std::string,std::string> name_and_email = get_email_address(info.signers[0].name);
   g_snprintf(buf, sizeof(buf),
              _("<u>This is a <b>PGP-Signed</b> message.</u>\n\n"
-             "<b>Signer:</b> %s (“%s”)\n"
+             "<b>Signer:</b> %s ('%s')\n"
              "<b>Valid until:</b> %s\n"
              "<b>Created on:</b> %s"),
              name_and_email.first.c_str(), name_and_email.second.c_str(),
@@ -1371,7 +1345,7 @@ BodyPane :: clear ()
 {
   if (_message)
     g_object_unref (_message);
-  _message = 0;
+  _message = nullptr;
 
   set_cleared(true);
 
@@ -1530,6 +1504,7 @@ BodyPane :: copy_url_cb (GtkMenuItem *mi G_GNUC_UNUSED, gpointer pane)
 {
   static_cast<BodyPane*>(pane)->copy_url ();
 }
+
 void
 BodyPane :: copy_url ()
 {
@@ -1537,7 +1512,6 @@ BodyPane :: copy_url ()
                           _hover_url.c_str(), _hover_url.size());
 }
 
-
 void
 BodyPane :: populate_popup_cb (GtkTextView *v, GtkMenu *m, gpointer pane)
 {
@@ -1577,9 +1551,9 @@ namespace
     if (event->type == GDK_BUTTON_PRESS && event->button == 3)
     {
       gtk_menu_popup
-      (GTK_MENU(bp->_menu), NULL, NULL, NULL, NULL,
-      (event ? event->button : 0),
-      (event ? event->time : 0));
+      (GTK_MENU(bp->_menu), nullptr, nullptr, nullptr, nullptr,
+      event->button,
+      event->time);
     }
 
     return TRUE;
@@ -1621,7 +1595,7 @@ BodyPane :: menu_clicked_all_cb (GtkWidget* w, gpointer ptr)
 GtkWidget*
 BodyPane :: new_attachment (const char* filename)
 {
-  if (!filename) return 0;
+  if (!filename) return nullptr;
 
   GtkWidget* w = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
   GtkWidget* attachment = gtk_label_new(filename);
@@ -1648,13 +1622,13 @@ BodyPane :: clear_attachments()
   _cur_col = 0;
   _cur_row = 0;
   _attachments = 0;
-  _current_attachment = 0;
+  _current_attachment = nullptr;
 
   {
     gtk_widget_set_no_show_all (_att_box, TRUE);
     gtk_widget_hide (_att_box);
     gtk_container_remove (GTK_CONTAINER (_att_box), _att_toolbar);
-    _att_toolbar = NULL;
+    _att_toolbar = nullptr;
     (void)create_attachments_toolbar(_att_box);
   }
 
@@ -1671,21 +1645,6 @@ BodyPane :: add_attachment_to_toolbar (const char* fn)
 //  gtk_widget_set_size_request(w, -1, 32);
 
   ++_attachments;
-#if !GTK_CHECK_VERSION(3,0,0)
-
-  const guint cols(4);
-
-  if (_attachments % 4 == 0 && _attachments != 0)
-  {
-    ++_cur_row;
-    gtk_table_resize (GTK_TABLE(_att_toolbar), _cur_row, cols);
-    _cur_col = 0;
-  }
-
-  gtk_table_attach_defaults (GTK_TABLE(_att_toolbar), w, _cur_col, _cur_col+1, _cur_row,_cur_row+1);
-
-  ++_cur_col;
-#else
   if (_attachments % 4 == 0 && _attachments != 0)
   {
     gtk_grid_insert_row (GTK_GRID(_att_toolbar), ++_cur_row);
@@ -1693,7 +1652,6 @@ BodyPane :: add_attachment_to_toolbar (const char* fn)
   }
 
   gtk_grid_attach (GTK_GRID(_att_toolbar), w, _cur_col++, _cur_row, 1, 1);
-#endif  // 3.0.0
 
   gtk_widget_set_no_show_all (_att_box, FALSE);
   gtk_widget_show_all (_att_box);
@@ -1703,7 +1661,7 @@ BodyPane :: add_attachment_to_toolbar (const char* fn)
 void
 BodyPane :: set_html_text (const char* text)
 {
-  webkit_web_view_load_string (WEBKIT_WEB_VIEW (_web_view), text, NULL, NULL, "");
+  webkit_web_view_load_string (WEBKIT_WEB_VIEW (_web_view), text, nullptr, nullptr, "");
 }
 #endif
 
@@ -1715,16 +1673,10 @@ BodyPane :: create_attachments_toolbar (GtkWidget* box)
   _cur_row = 0;
 
   GtkWidget * w;
-#if !GTK_CHECK_VERSION(3,0,0)
-  w = _att_toolbar = gtk_table_new(1,4,TRUE);
-  gtk_table_set_col_spacings (GTK_TABLE(w), PAD);
-  gtk_table_set_row_spacings (GTK_TABLE(w), PAD);
-#else
   w = _att_toolbar = gtk_grid_new();
   gtk_grid_insert_row (GTK_GRID(w), 0);
   gtk_grid_set_column_spacing (GTK_GRID(w), 3);
   gtk_grid_set_row_spacing (GTK_GRID (w), 4);
-#endif
   gtk_container_add (GTK_CONTAINER (box), w);
 
   return box;
@@ -1741,31 +1693,31 @@ BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs, GroupPrefs
   _header_pane(hp),
   _data (data),
   _cache (cache),
+#ifdef HAVE_WEBKIT
+  _web_view (webkit_web_view_new ()),
+#endif
   _hscroll_visible (false),
   _vscroll_visible (false),
-  _message (0),
+  _message (nullptr),
 #ifdef HAVE_GMIME_CRYPTO
 //  _gpgerr(GPG_DECODE),
 #endif
   _attachments(0),
-  _current_attachment(0),
-  _cleared (true)
-#ifdef HAVE_WEBKIT
-  ,_web_view (webkit_web_view_new ())
-#endif
+  _cleared (true),
+  _current_attachment(nullptr)
 {
 
   GtkWidget * w, * l, * hbox;
 
   for (guint i=0; i<NUM_ICONS; ++i)
-    icons[i].pixbuf = gdk_pixbuf_new_from_inline (-1, icons[i].pixbuf_txt, FALSE, 0);
+    icons[i].pixbuf = load_icon (icons[i].pixbuf_file);
 
   create_cursors();
   create_emoticons();
 
   // menu for popup menu for attachments
   _menu = gtk_menu_new ();
-  l = gtk_menu_item_new_with_label(_("Save Attachment As…"));
+  l = gtk_menu_item_new_with_label(_("Save Attachment As..."));
   g_signal_connect (l, "activate", G_CALLBACK(menu_clicked_as_cb), this);
   gtk_menu_shell_append (GTK_MENU_SHELL(_menu), l);
   l =  gtk_menu_item_new_with_label(_("Save All Attachments"));
@@ -1785,15 +1737,11 @@ BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs, GroupPrefs
   // tell the expander that it _wants_ to be very small, then it will still take
   // extra space given to it by its parent without asking for enough size to
   // fit the entire label.
-  w = _expander = gtk_expander_new (NULL);
+  w = _expander = gtk_expander_new (nullptr);
   gtk_widget_set_size_request (w, 50, -1);
   g_signal_connect (w, "activate", G_CALLBACK(expander_activated_cb), this);
   gtk_box_pack_start (GTK_BOX(vbox), w, false, false, 0);
-#if !GTK_CHECK_VERSION(3,0,0)
-  gtk_box_pack_start (GTK_BOX(vbox), gtk_hseparator_new(), false, false, 0);
-#else
   gtk_box_pack_start (GTK_BOX(vbox), gtk_separator_new(GTK_ORIENTATION_HORIZONTAL), false, false, 0);
-#endif
 
   _terse = gtk_label_new ("Expander");
   g_object_ref_sink (G_OBJECT(_terse));
@@ -1833,7 +1781,7 @@ BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs, GroupPrefs
   gtk_container_set_border_width (GTK_CONTAINER(_text), PAD_SMALL);
   gtk_text_view_set_editable (GTK_TEXT_VIEW(_text), false);
   gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW(_text), false);
-  _scroll = gtk_scrolled_window_new (NULL, NULL);
+  _scroll = gtk_scrolled_window_new (nullptr, nullptr);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (_scroll),
                                   GTK_POLICY_AUTOMATIC,
                                   GTK_POLICY_AUTOMATIC);
@@ -1846,24 +1794,16 @@ BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs, GroupPrefs
   GtkWidget * att_label = gtk_label_new (_("Attachments:"));
   gtk_misc_set_padding (GTK_MISC(att_label), PAD_SMALL, 0);
   gtk_misc_set_alignment (GTK_MISC(att_label), 0, 0);
-#if !GTK_CHECK_VERSION(3,0,0)
-  gtk_box_pack_start (GTK_BOX(_att_box), gtk_hseparator_new(), false, false, 0);
-#else
   gtk_box_pack_start (GTK_BOX(_att_box), gtk_separator_new(GTK_ORIENTATION_HORIZONTAL), false, false, 0);
-#endif
   gtk_box_pack_start (GTK_BOX(_att_box), att_label, false, false, 0);
   gtk_box_pack_start (GTK_BOX(vbox), create_attachments_toolbar (_att_box), false, false, 0);
-#if !GTK_CHECK_VERSION(3,0,0)
-  gtk_box_pack_start (GTK_BOX(vbox), gtk_hseparator_new(), false, false, 0);
-#else
   gtk_box_pack_start (GTK_BOX(vbox), gtk_separator_new(GTK_ORIENTATION_HORIZONTAL), false, false, 0);
-#endif
 
   // set up the buffer tags
   _buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW(_text));
   set_text_buffer_tags (_buffer, _prefs);
 
-  set_text_from_message (0);
+  set_text_from_message (nullptr);
   const bool expanded (_prefs.get_flag ("body-pane-headers-expanded", true));
   gtk_expander_set_expanded (GTK_EXPANDER(_expander), expanded);
   expander_activated_idle (this);
@@ -1874,7 +1814,7 @@ BodyPane :: BodyPane (Data& data, ArticleCache& cache, Prefs& prefs, GroupPrefs
   gtk_notebook_append_page (n, vbox, gtk_label_new (_("Text View")));
 
   // add scroll to html
-  GtkWidget* scroll = gtk_scrolled_window_new (NULL, NULL);
+  GtkWidget* scroll = gtk_scrolled_window_new (nullptr, nullptr);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(scroll), GTK_SHADOW_IN);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scroll),
                                   GTK_POLICY_AUTOMATIC,
@@ -2045,7 +1985,7 @@ namespace
 GMimeMessage*
 BodyPane :: create_followup_or_reply (bool is_reply)
 {
-  GMimeMessage * msg (0);
+  GMimeMessage * msg (nullptr);
 
   if (_message)
   {
@@ -2056,7 +1996,7 @@ BodyPane :: create_followup_or_reply (bool is_reply)
     // fallback character encodings
     const char * group_charset (_charset.c_str());
     GMimeContentType * type (g_mime_object_get_content_type (GMIME_OBJECT(_message)));
-    const char * message_charset (type ? g_mime_content_type_get_parameter (type, "charset") : 0);
+    const char * message_charset (type ? g_mime_content_type_get_parameter (type, "charset") : nullptr);
 
     ///
     ///  HEADERS
@@ -2070,18 +2010,10 @@ BodyPane :: create_followup_or_reply (bool is_reply)
 
     if (is_reply || fup_to=="poster") {
       const std::string& to (reply_to.empty() ? from : reply_to);
-#ifdef HAVE_GMIME_30
       pan_g_mime_message_add_recipients_from_string (msg, GMIME_ADDRESS_TYPE_TO, to.c_str());
-#else
-      pan_g_mime_message_add_recipients_from_string (msg, GMIME_RECIPIENT_TYPE_TO, to.c_str());
-#endif
     } else {
       const std::string& groups (fup_to.empty() ? newsgroups : fup_to);
-#ifdef HAVE_GMIME_30
-      g_mime_object_append_header ((GMimeObject *) msg, "Newsgroups", groups.c_str(), NULL);
-#else
-      g_mime_object_append_header ((GMimeObject *) msg, "Newsgroups", groups.c_str());
-#endif
+      g_mime_object_append_header ((GMimeObject *) msg, "Newsgroups", groups.c_str(), nullptr);
     }
 
     // Subject:
@@ -2090,43 +2022,22 @@ BodyPane :: create_followup_or_reply (bool is_reply)
     std::string val (normalize_subject_re (h));
     if (val.find ("Re:") != 0) // add "Re: " if we don't have one
       val.insert (0, "Re: ");
-#ifdef HAVE_GMIME_30
-    g_mime_message_set_subject (msg, val.c_str(), NULL);
-#else
-    g_mime_message_set_subject (msg, val.c_str());
-#endif
+    g_mime_message_set_subject (msg, val.c_str(), nullptr);
 
     // attribution lines
     const char * cpch = g_mime_object_get_header (_message_obj, "From");
     h = header_to_utf8 (cpch, message_charset, group_charset);
-#ifdef HAVE_GMIME_30
-    g_mime_object_append_header (msg_obj, "X-Draft-Attribution-Author", h.c_str(), NULL);
-#else
-    g_mime_object_append_header (msg_obj, "X-Draft-Attribution-Author", h.c_str());
-#endif
+    g_mime_object_append_header (msg_obj, "X-Draft-Attribution-Author", h.c_str(), nullptr);
 
     cpch = g_mime_message_get_message_id (_message);
     h = header_to_utf8 (cpch, message_charset, group_charset);
-#ifdef HAVE_GMIME_30
-    g_mime_object_append_header (msg_obj, "X-Draft-Attribution-Id", h.c_str(), NULL);
-#else    
-    g_mime_object_append_header (msg_obj, "X-Draft-Attribution-Id", h.c_str());
-#endif    
+    g_mime_object_append_header (msg_obj, "X-Draft-Attribution-Id", h.c_str(), nullptr);
 
-#ifdef HAVE_GMIME_30
     const char * header_t = "Date";
     const char * tmp_s = g_mime_object_get_header (_message_obj, header_t);
     const char * tmp = tmp_s;  // FIXME: convert time to string
-#else
-    char * tmp = g_mime_message_get_date_as_string (_message);
-#endif
     h = header_to_utf8 (tmp, message_charset, group_charset);
-#ifdef HAVE_GMIME_30
-    g_mime_object_append_header (msg_obj, "X-Draft-Attribution-Date", h.c_str(), NULL);
-#else
-    g_mime_object_append_header (msg_obj, "X-Draft-Attribution-Date", h.c_str());
-    g_free (tmp);
-#endif
+    g_mime_object_append_header (msg_obj, "X-Draft-Attribution-Date", h.c_str(), nullptr);
 
     // references
     const char * header = "References";
@@ -2138,11 +2049,7 @@ BodyPane :: create_followup_or_reply (bool is_reply)
     val += g_mime_message_get_message_id (_message);
     val += ">";
     val = GNKSA :: trim_references (val);
-#ifdef HAVE_GMIME_30
-    g_mime_object_append_header (msg_obj, header, val.c_str(), NULL);
-#else    
-    g_mime_object_append_header (msg_obj, header, val.c_str());
-#endif    
+    g_mime_object_append_header (msg_obj, header, val.c_str(), nullptr);
 
     ///
     ///  BODY
@@ -2180,17 +2087,9 @@ BodyPane :: create_followup_or_reply (bool is_reply)
     g_mime_stream_write_string (stream, s.c_str());
     GMimeDataWrapper * wrapper = g_mime_data_wrapper_new_with_stream (stream, GMIME_CONTENT_ENCODING_8BIT);
     GMimePart * part = g_mime_part_new ();
-#ifdef HAVE_GMIME_30
-    GMimeContentType * new_type = g_mime_content_type_parse (NULL, "text/plain; charset=UTF-8");
-#else    
-    GMimeContentType * new_type = g_mime_content_type_new_from_string ("text/plain; charset=UTF-8");
-#endif    
+    GMimeContentType * new_type = g_mime_content_type_parse (nullptr, "text/plain; charset=UTF-8");
     g_mime_object_set_content_type ((GMimeObject *) part, new_type);
-#ifdef HAVE_GMIME_30
     g_mime_part_set_content (part, wrapper);
-#else    
-    g_mime_part_set_content_object (part, wrapper);
-#endif    
     g_mime_part_set_content_encoding (part, GMIME_CONTENT_ENCODING_8BIT);
     g_mime_message_set_mime_part (msg, GMIME_OBJECT(part));
     g_object_unref (new_type);
@@ -2216,7 +2115,7 @@ BodyPane :: refresh_fonts ()
   const bool monospace_font_enabled = _prefs.get_flag ("monospace-font-enabled", false);
 
   if (!body_pane_font_enabled && !monospace_font_enabled)
-    gtk_widget_override_font (_text, 0);
+    gtk_widget_override_font (_text, nullptr);
   else {
     const std::string str (monospace_font_enabled
       ? _prefs.get_string ("monospace-font", "Monospace 10")
@@ -2252,13 +2151,13 @@ BodyPane :: on_prefs_string_changed (const StringView& key, const StringView& va
 }
 
 void
-BodyPane :: on_prefs_color_changed (const StringView& key, const GdkColor& color G_GNUC_UNUSED)
-{		
-  if ((key == "text-color-fg")              || (key == "text-color-bg")              || 
-      (key == "body-pane-color-url")        || (key == "body-pane-color-url-bg")     || 
-      (key == "body-pane-color-quote-1")    || (key == "body-pane-color-quote-2")    || 
-      (key == "body-pane-color-quote-3")    || (key == "body-pane-color-quote-1-bg") || 
-      (key == "body-pane-color-quote-2-bg") || (key == "body-pane-color-quote-3-bg") || 
+BodyPane :: on_prefs_color_changed (const StringView& key, const GdkRGBA& color G_GNUC_UNUSED)
+{
+  if ((key == "text-color-fg")              || (key == "text-color-bg")              ||
+      (key == "body-pane-color-url")        || (key == "body-pane-color-url-bg")     ||
+      (key == "body-pane-color-quote-1")    || (key == "body-pane-color-quote-2")    ||
+      (key == "body-pane-color-quote-3")    || (key == "body-pane-color-quote-1-bg") ||
+      (key == "body-pane-color-quote-2-bg") || (key == "body-pane-color-quote-3-bg") ||
       (key == "body-pane-color-signature")  || (key == "body-pane-color-signature-bg"))
   {
     refresh_colors ();
@@ -2282,3 +2181,5 @@ BodyPane :: set_character_encoding (const char * charset)
 
   refresh ();
 }
+
+}
