@@ -19,7 +19,8 @@
 
 #include <config.h>
 #include <glib/gi18n.h>
-#include "gtk-compat.h"
+#include <gtk/gtk.h>
+#include <gdk/gdk.h>
 #include <cctype>
 #include <cmath>
 #include <algorithm>
@@ -34,11 +35,11 @@
 #include <pan/usenet-utils/mime-utils.h>
 #include <pan/data/article.h>
 #include <pan/data/data.h>
-#include <pan/icons/pan-pixbufs.h>
 #include "header-pane.h"
+#include "pan/gui/load-icon.h"
 #include "render-bytes.h"
 #include "tango-colors.h"
-#include "gtk-compat.h"
+
 
 using namespace pan;
 
@@ -68,7 +69,7 @@ const Article*
 HeaderPane :: get_article (GtkTreeModel* model, GtkTreeIter* iter)
 {
   const Article * a = dynamic_cast<Row*>(PAN_TREE_STORE(model)->get_row(iter))->article;
-  g_assert (a != 0);
+  g_assert (a != nullptr);
   return a;
 }
 
@@ -101,24 +102,24 @@ namespace
    };
 
   struct Icon {
-    const guint8 * pixbuf_txt;
+    const char * icon_file;
     GdkPixbuf * pixbuf;
   } _icons[ICON_QTY] = {
-    { icon_article_read,           0 },
-    { icon_article_unread,         0 },
+    { "icon_article_read.png",           nullptr },
+    { "icon_article_unread.png",         nullptr },
 
-    { icon_binary_complete,        0 },
-    { icon_binary_complete_read,   0 },
+    { "icon_binary_complete.png",        nullptr },
+    { "icon_binary_complete_read.png",   nullptr },
 
-    { icon_binary_incomplete,      0 },
-    { icon_binary_incomplete_read, 0 },
+    { "icon_binary_incomplete.png",      nullptr },
+    { "icon_binary_incomplete_read.png", nullptr },
 
-    { icon_disk,                   0 },
-    { icon_bluecheck,              0 },
-    { icon_x,                      0 },
-    { icon_empty,                  0 },
-    { icon_red_flag,               0 },
-    { icon_get_flagged,            0 }
+    { "icon_disk.png",                   nullptr },
+    { "icon_bluecheck.png",              nullptr },
+    { "icon_x.png",                      nullptr },
+    { "icon_empty.png",                  nullptr },
+    { "icon_red_flag.png",               nullptr },
+    { "icon_get_flagged.png",            nullptr }
   };
 
   int
@@ -175,7 +176,7 @@ HeaderPane :: render_action (GtkTreeViewColumn * ,
 {
   int index (0);
   gtk_tree_model_get (model, iter, COL_ACTION, &index, -1);
-  g_object_set (renderer, "pixbuf", _icons[index].pixbuf, NULL);
+  g_object_set (renderer, "pixbuf", _icons[index].pixbuf, nullptr);
 }
 
 void
@@ -187,7 +188,7 @@ HeaderPane :: render_state (GtkTreeViewColumn * ,
 {
   int index (0);
   gtk_tree_model_get (model, iter, COL_STATE, &index, -1);
-  g_object_set (renderer, "pixbuf", _icons[index].pixbuf, NULL);
+  g_object_set (renderer, "pixbuf", _icons[index].pixbuf, nullptr);
 }
 
 int
@@ -253,9 +254,9 @@ HeaderPane :: render_score (GtkTreeViewColumn * ,
   }
 
   g_object_set (renderer, "text", buf,
-                          "background", (bg.empty() ? NULL : bg.c_str()),
-                          "foreground", (fg.empty() ? NULL : fg.c_str()),
-                          NULL);
+                          "background", (bg.empty() ? nullptr : bg.c_str()),
+                          "foreground", (fg.empty() ? nullptr : fg.c_str()),
+                          nullptr);
 }
 
 void
@@ -265,18 +266,13 @@ HeaderPane :: render_author (GtkTreeViewColumn * ,
                             GtkTreeIter       * iter,
                             gpointer            user_data)
 {
-
   const HeaderPane * self (static_cast<HeaderPane*>(user_data));
-  const Row * row (dynamic_cast<Row*>(self->_tree_store->get_row (iter)));
-
   const Article * a (self->get_article (model, iter));
 
-  char* ret = __g_mime_iconv_strdup(conv, a->author.c_str());
-  if (ret) g_object_set (renderer, "text", ret,
+  g_object_set (renderer, "text", a->author.c_str() ,
                          "background", self->_bg.c_str(),
                          "foreground", self->_fg.c_str(),
-                         NULL);
-  g_free(ret);
+                         nullptr);
 }
 
 void
@@ -298,7 +294,7 @@ HeaderPane :: render_lines (GtkTreeViewColumn * ,
   g_object_set (renderer, "text", str.str().c_str(),
                 "background", self->_bg.c_str(),
                 "foreground", self->_fg.c_str(),
-                NULL);
+                nullptr);
 }
 
 void
@@ -316,7 +312,7 @@ HeaderPane :: render_bytes (GtkTreeViewColumn * ,
                 "text", pan::render_bytes(bytes),
                 "background", self->_bg.c_str(),
                 "foreground", self->_fg.c_str(),
-                NULL);
+                nullptr);
 }
 
 void
@@ -328,13 +324,13 @@ HeaderPane :: render_date  (GtkTreeViewColumn * ,
 {
   const HeaderPane * self (static_cast<HeaderPane*>(userdata));
 
-  gchar* date (0);
+  gchar* date (nullptr);
   gtk_tree_model_get (model, iter, COL_DATE_STR, &date, -1);
   g_object_set (renderer,
                 "text", date,
                 "background", self->_bg.c_str(),
                 "foreground", self->_fg.c_str(),
-                NULL);
+                nullptr);
   g_free (date);
 }
 
@@ -374,10 +370,7 @@ HeaderPane :: render_subject (GtkTreeViewColumn * ,
 
   const Article * a (self->get_article (model, iter));
 
-  char* ret = __g_mime_iconv_strdup(conv, a->subject.c_str());
-  std::string res;
-  if (ret) res = ret;
-  g_free(ret);
+  std::string res = a->subject.c_str();
 
   char buf[512];
 
@@ -406,7 +399,7 @@ HeaderPane :: render_subject (GtkTreeViewColumn * ,
     "text", res.c_str(),
     "weight", (bold ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL),
     "foreground", unread ? (def_fg.empty() ? self->_fg.c_str() : def_fg.c_str()) : self->_fg.c_str(),
-    "background", unread ? (def_bg.empty() ? self->_bg.c_str() : def_bg.c_str()) : self->_bg.c_str(), NULL);
+    "background", unread ? (def_bg.empty() ? self->_bg.c_str() : def_bg.c_str()) : self->_bg.c_str(), nullptr);
 
 }
 
@@ -414,7 +407,7 @@ HeaderPane::Row*
 HeaderPane :: get_row (const Quark& message_id)
 {
   mid_to_row_t::iterator it (_mid_to_row.find (message_id));
-  return it==_mid_to_row.end() ? 0 : *it;
+  return it==_mid_to_row.end() ? nullptr : *it;
 }
 
 HeaderPane::Row*
@@ -451,7 +444,7 @@ HeaderPane ::  add_children_to_model (PanTreeStore               * store,
   rows.reserve (children.size());
   foreach_const (article_v, children, it)
     rows.push_back (create_row (date_maker, *it));
-  store->append (do_thread ? parent_row : 0, rows);
+  store->append (do_thread ? parent_row : nullptr, rows);
 
   // recurse
   for (size_t i=0, n=children.size(); i<n; ++i)
@@ -501,7 +494,7 @@ HeaderPane :: column_compare_func (GtkTreeModel  * model,
     case COL_BYTES: {
       const unsigned long a_bytes (row_a.article->get_byte_count());
       const unsigned long b_bytes (row_b.article->get_byte_count());
-           if (a_bytes < b_bytes) ret = -1;
+      if      (a_bytes < b_bytes) ret = -1;
       else if (a_bytes > b_bytes) ret = 1;
       else                        ret = 0;
       break;
@@ -510,7 +503,7 @@ HeaderPane :: column_compare_func (GtkTreeModel  * model,
     case COL_LINES: {
       const unsigned long a_lines (row_a.article->get_line_count());
       const unsigned long b_lines (row_b.article->get_line_count());
-           if (a_lines < b_lines) ret = -1;
+      if      (a_lines < b_lines) ret = -1;
       else if (a_lines > b_lines) ret = 1;
       else                        ret = 0;
       break;
@@ -523,7 +516,7 @@ HeaderPane :: column_compare_func (GtkTreeModel  * model,
     default: { // COL_DATE
       const time_t a_time (row_a.article->time_posted);
       const time_t b_time (row_b.article->time_posted);
-           if (a_time < b_time) ret = -1;
+      if      (a_time < b_time) ret = -1;
       else if (a_time > b_time) ret = 1;
       else                      ret = 0;
       break;
@@ -571,11 +564,11 @@ HeaderPane :: build_model (const Quark               & group,
 
   GtkTreeSortable * sort = GTK_TREE_SORTABLE(store);
   for (int i=0; i<N_COLUMNS; ++i)
-    gtk_tree_sortable_set_sort_func (sort, i, column_compare_func, GINT_TO_POINTER(i), NULL);
+    gtk_tree_sortable_set_sort_func (sort, i, column_compare_func, GINT_TO_POINTER(i), nullptr);
   if (!group.empty()) {
     const EvolutionDateMaker date_maker;
     const bool do_thread (_prefs.get_flag ("thread-headers", true));
-    add_children_to_model (store, NULL, Quark(), atree, date_maker, do_thread);
+    add_children_to_model (store, nullptr, Quark(), atree, date_maker, do_thread);
   }
 
   return store;
@@ -608,7 +601,7 @@ HeaderPane :: rebuild ()
   }
 
   _mid_to_row.clear ();
-  _tree_store = build_model (_group, _atree, NULL);
+  _tree_store = build_model (_group, _atree, nullptr);
 
   const bool sort_ascending = _group_prefs.get_flag (_group, "header-pane-sort-ascending", false);
   int sort_column = _group_prefs.get_int (_group, "header-pane-sort-column", COL_DATE);
@@ -654,16 +647,16 @@ HeaderPane :: set_group (const Quark& new_group)
     {
       save_sort_order (get_group(), _group_prefs, _tree_store);
       _mid_to_row.clear ();
-      _tree_store = 0;
-      gtk_tree_view_set_model (GTK_TREE_VIEW(_tree_view), NULL);
+      _tree_store = nullptr;
+      gtk_tree_view_set_model (GTK_TREE_VIEW(_tree_view), nullptr);
     }
 
     _group = new_group;
 
     delete _atree;
-    _atree = 0;
+    _atree = nullptr;
 
-    char * pch = g_build_filename (g_get_home_dir(), "News", NULL);
+    char * pch = g_build_filename (g_get_home_dir(), "News", nullptr);
     Quark path(_group_prefs.get_string (_group, "default-group-save-path", pch));
     g_free(pch);
 
@@ -727,7 +720,7 @@ namespace
     const quarks_t& mids;
     ArticleIsNotInSet (const quarks_t& m): mids(m) {}
     virtual ~ArticleIsNotInSet () {}
-    virtual bool operator()(const Article& a) const {
+    virtual bool operator()(const Article& a) const override {
       return !mids.count(a.message_id);
     }
   };
@@ -737,7 +730,8 @@ namespace
     quarks_t& mids;
     RememberMessageId (quarks_t& m): mids(m) {}
     virtual ~RememberMessageId() {}
-    virtual void operator() (GtkTreeModel*, GtkTreeIter*, const Article& article) {
+    void operator() (GtkTreeModel*, GtkTreeIter*, const Article& article) override
+    {
       mids.insert (article.message_id);
     }
   };
@@ -748,7 +742,7 @@ HeaderPane :: collapse_selected()
 {
   {
   // get a list of paths
-  GtkTreeModel * model (0);
+  GtkTreeModel * model (nullptr);
   GtkTreeView * view (GTK_TREE_VIEW (_tree_view));
   GtkTreeSelection * selection (gtk_tree_view_get_selection (view));
   GList * list (gtk_tree_selection_get_selected_rows (selection, &model));
@@ -758,7 +752,7 @@ HeaderPane :: collapse_selected()
     gtk_tree_view_collapse_row (view, path);
 //    gtk_tree_view_expand_to_path (view, path);
   }
-  g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
+  g_list_foreach (list, (GFunc)gtk_tree_path_free, nullptr);
   g_list_free (list);
 }
 }
@@ -775,8 +769,8 @@ HeaderPane :: select_message_id (const Quark& mid, bool do_scroll)
   GtkTreeSelection * sel (gtk_tree_view_get_selection (view));
   gtk_tree_selection_select_path (sel, path);
   if (do_scroll) {
-    gtk_tree_view_set_cursor (view, path, NULL, false);
-    gtk_tree_view_scroll_to_cell (view, path, NULL, true, 0.5f, 0.0f);
+    gtk_tree_view_set_cursor (view, path, nullptr, false);
+    gtk_tree_view_scroll_to_cell (view, path, nullptr, true, 0.5f, 0.0f);
   }
   gtk_tree_path_free (path);
 }
@@ -801,7 +795,7 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
   if (!new_selection.empty()) {
     GtkTreeView *view (GTK_TREE_VIEW(_tree_view));
     Row * row (get_row (*new_selection.begin()));
-    GtkTreePath *a(0), *b(0), *p (_tree_store->get_path (row));
+    GtkTreePath *a(nullptr), *b(nullptr), *p (_tree_store->get_path (row));
     gtk_tree_view_get_visible_range (view, &a, &b);
     selection_was_visible = (gtk_tree_path_compare(a,p)<=0 &&
                              gtk_tree_path_compare(p,b)<=0);
@@ -827,13 +821,13 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
     foreach_const (Data::ArticleTree::Diffs::added_t, diffs.added, it)
       create_row (date_maker, _atree->get_article(it->first));
     foreach_const (Data::ArticleTree::Diffs::added_t, diffs.added, it) {
-      Row * parent (do_thread ? get_row (it->second.parent) : 0);
+      Row * parent (do_thread ? get_row (it->second.parent) : nullptr);
       Row * child (get_row (it->first));
       tmp[parent].push_back (child);
     }
 
     g_object_ref(G_OBJECT (_tree_store));
-    gtk_tree_view_set_model(GTK_TREE_VIEW(_tree_view), NULL);
+    gtk_tree_view_set_model(GTK_TREE_VIEW(_tree_view), nullptr);
     _tree_store->insert_sorted (tmp);
     gtk_tree_view_set_model(GTK_TREE_VIEW(_tree_view), GTK_TREE_MODEL (_tree_store));
     g_object_unref(G_OBJECT(_tree_store));
@@ -864,7 +858,7 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
     g_assert (keep.size() + kill.size() == _mid_to_row.size());
 
     g_object_ref(G_OBJECT (_tree_store));
-    gtk_tree_view_set_model(GTK_TREE_VIEW(_tree_view), NULL);
+    gtk_tree_view_set_model(GTK_TREE_VIEW(_tree_view), nullptr);
     _tree_store->remove (kill);
     gtk_tree_view_set_model(GTK_TREE_VIEW(_tree_view), GTK_TREE_MODEL (_tree_store));
     g_object_unref(G_OBJECT(_tree_store));
@@ -896,7 +890,7 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
 const Article*
 HeaderPane :: get_first_selected_article () const
 {
-   const Article * a (0);
+   const Article * a (nullptr);
    const std::set<const Article*> articles (get_full_selection ());
    if (!articles.empty())
      a = *articles.begin ();
@@ -906,7 +900,7 @@ HeaderPane :: get_first_selected_article () const
 Article*
 HeaderPane :: get_first_selected_article ()
 {
-   Article * a (0);
+   Article * a (nullptr);
    std::set<const Article*> articles (get_full_selection ());
    if (!articles.empty())
      a = (Article*)*articles.begin ();
@@ -1083,7 +1077,7 @@ HeaderPane ::  do_popup_menu (GtkWidget*, GdkEventButton *event, gpointer pane_g
 {
   HeaderPane * self (static_cast<HeaderPane*>(pane_g));
   GtkWidget * menu (self->_action_manager.get_action_widget ("/header-pane-popup"));
-  gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL,
+  gtk_menu_popup (GTK_MENU(menu), nullptr, nullptr, nullptr, nullptr,
                   (event ? event->button : 0),
                   (event ? event->time : 0));
 }
@@ -1092,7 +1086,7 @@ namespace
 {
   gboolean on_popup_menu (GtkWidget * treeview, gpointer userdata)
   {
-    HeaderPane::do_popup_menu (treeview, NULL, userdata);
+    HeaderPane::do_popup_menu (treeview, nullptr, userdata);
     return true;
   }
 }
@@ -1215,7 +1209,7 @@ HeaderPane :: on_button_pressed (GtkWidget * treeview, GdkEventButton *event, gp
     if (gtk_tree_view_get_path_at_pos (tv,
                                        (gint) event->x,
                                        (gint) event->y,
-                                       &path, NULL, NULL, NULL))
+                                       &path, nullptr, nullptr, nullptr))
     {
       if (!gtk_tree_selection_path_is_selected (selection, path))
       {
@@ -1269,14 +1263,18 @@ namespace
   gboolean on_row_activated_idle (gpointer pane_g)
   {
     HeaderPane * pane (static_cast<HeaderPane*>(pane_g));
-    const Article * a (pane->get_first_selected_article());
+    Article const *a(pane->get_first_selected_article());
     if (a) {
       const size_t lines = a->get_line_count();
-      const bool is_smallish = lines  <= 5000;
-      const bool is_mediumish = lines <= 20000;
-      const bool image_subject = has_image_type_in_subject (*a);
-      const bool is_pictures_newsgroup = pane->get_group().to_view().strstr("pictures")!=0;
-      if (is_smallish || image_subject)
+      bool const is_smallish = lines <= 5000;
+      bool const is_mediumish = lines <= 20000;
+      bool const image_subject = has_image_type_in_subject(*a);
+      bool const is_pictures_newsgroup =
+        pane->get_group().to_view().strstr("pictures") != nullptr;
+      // This is not the number of MIME parts
+      int const part_count = a->get_found_part_count();
+
+      if (is_smallish || image_subject || part_count < 2)
         pane->_action_manager.activate_action ("read-selected-article");
       else if (is_mediumish && is_pictures_newsgroup)
         pane->_action_manager.activate_action ("read-selected-article");
@@ -1574,11 +1572,7 @@ namespace
                                      GdkEventFocus * ,
                                      gpointer        )
   {
-#if !GTK_CHECK_VERSION(3,0,0)
-    gtk_widget_modify_text (w, GTK_STATE_NORMAL, NULL); // resets
-#else
-    gtk_widget_override_color (w, GTK_STATE_FLAG_NORMAL, NULL);
-#endif
+    gtk_widget_override_color (w, GTK_STATE_FLAG_NORMAL, nullptr);
     set_search_entry (w, search_text.c_str());
     return false;
   }
@@ -1587,16 +1581,9 @@ namespace
   {
     if (search_text.empty() && !gtk_widget_has_focus(w))
     {
-#if !GTK_CHECK_VERSION(3,0,0)
-      GdkColor c;
-      c.pixel = 0;
-      c.red = c.green = c.blue = 0xAAAA;
-      gtk_widget_modify_text (w, GTK_STATE_NORMAL, &c);
-#else
       GdkRGBA c;
       gdk_rgba_parse (&c, "0xAAA");
       gtk_widget_override_color(w, GTK_STATE_FLAG_NORMAL, &c);
-#endif
       set_search_entry (w, _(mode_strings[mode]));
     }
   }
@@ -1671,7 +1658,7 @@ namespace
   void entry_icon_release (GtkEntry*, GtkEntryIconPosition icon_pos, GdkEventButton*, gpointer menu)
   {
     if (icon_pos == GTK_ENTRY_ICON_PRIMARY)
-      gtk_menu_popup (GTK_MENU(menu), 0, 0, 0, 0, 0, gtk_get_current_event_time());
+      gtk_menu_popup (GTK_MENU(menu), nullptr, nullptr, nullptr, nullptr, 0, gtk_get_current_event_time());
   }
 
   void entry_icon_release_2 (GtkEntry *entry, GtkEntryIconPosition icon_pos, GdkEventButton*, gpointer pane_gpointer)
@@ -1680,14 +1667,14 @@ namespace
       set_search_entry (GTK_WIDGET(entry), "");
       refresh_search_entry (GTK_WIDGET(entry));
       search_text.clear ();
-      search_entry_activated (NULL, pane_gpointer);
+      search_entry_activated (nullptr, pane_gpointer);
     }
   }
 
 
   void ellipsize_if_supported (GObject * o)
   {
-    g_object_set (o, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+    g_object_set (o, "ellipsize", PANGO_ELLIPSIZE_END, nullptr);
   }
 }
 
@@ -1699,7 +1686,7 @@ HeaderPane :: build_tree_columns ()
 
   // out with the old columns, if any
   GList * old_columns = gtk_tree_view_get_columns (tree_view);
-  for (GList *l=old_columns; l!=NULL; l=l->next)
+  for (GList *l=old_columns; l!=nullptr; l=l->next)
     gtk_tree_view_remove_column (tree_view, GTK_TREE_VIEW_COLUMN(l->data));
   g_list_free (old_columns);
 
@@ -1710,20 +1697,20 @@ HeaderPane :: build_tree_columns ()
   {
     const std::string& name (tok.to_string());
     const std::string width_key = std::string("header-pane-") + name +  "-column-width";
-    GtkTreeViewColumn * col (0);
+    GtkTreeViewColumn * col (nullptr);
 
     if (name == "state")
     {
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_PIXBUF,
         "xpad", xpad,
         "ypad", 0,
-        NULL));
+        nullptr));
       col = gtk_tree_view_column_new ();
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 24));
       gtk_tree_view_column_set_resizable (col, false);
       gtk_tree_view_column_pack_start (col, r, false);
-      gtk_tree_view_column_set_cell_data_func (col, r, render_state, 0, 0);
+      gtk_tree_view_column_set_cell_data_func (col, r, render_state, nullptr, nullptr);
       gtk_tree_view_column_set_sort_column_id (col, COL_STATE);
       gtk_tree_view_append_column (tree_view, col);
     }
@@ -1732,13 +1719,13 @@ HeaderPane :: build_tree_columns ()
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_PIXBUF,
         "xpad", xpad,
         "ypad", 0,
-        NULL));
+        nullptr));
       col = gtk_tree_view_column_new ();
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 24));
       gtk_tree_view_column_set_resizable (col, false);
       gtk_tree_view_column_pack_start (col, r, false);
-      gtk_tree_view_column_set_cell_data_func (col, r, render_action, 0, 0);
+      gtk_tree_view_column_set_cell_data_func (col, r, render_action, nullptr, nullptr);
       gtk_tree_view_append_column (tree_view, col);
     }
     else if (name == "subject")
@@ -1746,14 +1733,14 @@ HeaderPane :: build_tree_columns ()
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
         "xpad", xpad,
         "ypad", 0,
-        NULL));
+        nullptr));
       ellipsize_if_supported (G_OBJECT(r));
-      col = gtk_tree_view_column_new_with_attributes (_("Subject"), r, NULL);
+      col = gtk_tree_view_column_new_with_attributes (_("Subject"), r, nullptr);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 400));
       gtk_tree_view_column_set_resizable (col, true);
       gtk_tree_view_column_set_sort_column_id (col, COL_SUBJECT);
-      gtk_tree_view_column_set_cell_data_func (col, r, render_subject, this, 0);
+      gtk_tree_view_column_set_cell_data_func (col, r, render_subject, this, nullptr);
       gtk_tree_view_append_column (tree_view, col);
       gtk_tree_view_set_expander_column (tree_view, col);
     }
@@ -1763,14 +1750,14 @@ HeaderPane :: build_tree_columns ()
         "xpad", xpad,
         "ypad", 0,
         "xalign", 1.0,
-        NULL));
+        nullptr));
       ellipsize_if_supported (G_OBJECT(r));
-      col = gtk_tree_view_column_new_with_attributes (_("Score"), r, NULL);
+      col = gtk_tree_view_column_new_with_attributes (_("Score"), r, nullptr);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 50));
       gtk_tree_view_column_set_resizable (col, true);
       gtk_tree_view_column_set_sort_column_id (col, COL_SCORE);
-      gtk_tree_view_column_set_cell_data_func (col, r, render_score, this, 0);
+      gtk_tree_view_column_set_cell_data_func (col, r, render_score, this, nullptr);
       gtk_tree_view_append_column (tree_view, col);
     }
     else if (name == "author")
@@ -1778,14 +1765,14 @@ HeaderPane :: build_tree_columns ()
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
         "xpad", xpad,
         "ypad", 0,
-        NULL));
+        nullptr));
       ellipsize_if_supported (G_OBJECT(r));
-      col = gtk_tree_view_column_new_with_attributes (_("Author"), r, NULL);
+      col = gtk_tree_view_column_new_with_attributes (_("Author"), r, nullptr);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 133));
       gtk_tree_view_column_set_resizable (col, true);
       gtk_tree_view_column_set_sort_column_id (col, COL_SHORT_AUTHOR);
-      gtk_tree_view_column_set_cell_data_func (col, r, render_author, this, 0);
+      gtk_tree_view_column_set_cell_data_func (col, r, render_author, this, nullptr);
       gtk_tree_view_append_column (tree_view, col);
     }
     else if (name == "lines")
@@ -1794,14 +1781,14 @@ HeaderPane :: build_tree_columns ()
         "xpad", xpad,
         "ypad", 0,
         "xalign", 1.0,
-        NULL));
+        nullptr));
       ellipsize_if_supported (G_OBJECT(r));
-      col = gtk_tree_view_column_new_with_attributes (_("Lines"), r, NULL);
+      col = gtk_tree_view_column_new_with_attributes (_("Lines"), r, nullptr);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 60));
       gtk_tree_view_column_set_resizable (col, true);
       gtk_tree_view_column_set_sort_column_id (col, COL_LINES);
-      gtk_tree_view_column_set_cell_data_func (col, r, render_lines, this, 0);
+      gtk_tree_view_column_set_cell_data_func (col, r, render_lines, this, nullptr);
       gtk_tree_view_append_column (tree_view, col);
     }
     else if (name == "bytes")
@@ -1810,14 +1797,14 @@ HeaderPane :: build_tree_columns ()
         "xpad", xpad,
         "ypad", 0,
         "xalign", 1.0,
-        NULL));
+        nullptr));
       ellipsize_if_supported (G_OBJECT(r));
-      col = gtk_tree_view_column_new_with_attributes (_("Bytes"), r, NULL);
+      col = gtk_tree_view_column_new_with_attributes (_("Bytes"), r, nullptr);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 80));
       gtk_tree_view_column_set_resizable (col, true);
       gtk_tree_view_column_set_sort_column_id (col, COL_BYTES);
-      gtk_tree_view_column_set_cell_data_func (col, r, render_bytes, this, 0);
+      gtk_tree_view_column_set_cell_data_func (col, r, render_bytes, this, nullptr);
       gtk_tree_view_append_column (tree_view, col);
     }
     else if (name == "date")
@@ -1825,14 +1812,14 @@ HeaderPane :: build_tree_columns ()
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
         "xpad", xpad,
         "ypad", 0,
-        NULL));
+        nullptr));
       ellipsize_if_supported (G_OBJECT(r));
-      col = gtk_tree_view_column_new_with_attributes (_("Date"), r, NULL);
+      col = gtk_tree_view_column_new_with_attributes (_("Date"), r, nullptr);
       gtk_tree_view_column_set_sizing (col, GTK_TREE_VIEW_COLUMN_FIXED);
       gtk_tree_view_column_set_fixed_width (col, _prefs.get_int (width_key, 120));
       gtk_tree_view_column_set_resizable (col, true);
       gtk_tree_view_column_set_sort_column_id (col, COL_DATE);
-      gtk_tree_view_column_set_cell_data_func (col, r, render_date, this, 0);
+      gtk_tree_view_column_set_cell_data_func (col, r, render_date, this, nullptr);
       gtk_tree_view_append_column (tree_view, col);
     }
 
@@ -1870,7 +1857,7 @@ HeaderPane :: ~HeaderPane ()
 
   // save the column widths
   GList * columns = gtk_tree_view_get_columns (GTK_TREE_VIEW(_tree_view));
-  for (GList *l=columns; l!=NULL; l=l->next) {
+  for (GList *l=columns; l!=nullptr; l=l->next) {
     GtkTreeViewColumn * col (GTK_TREE_VIEW_COLUMN(l->data));
     const int width = gtk_tree_view_column_get_width (col);
     const char * width_key = (const char*) g_object_get_data (G_OBJECT(col), "column-width-key");
@@ -1892,17 +1879,17 @@ HeaderPane :: create_filter_entry ()
   GtkWidget * entry = gtk_entry_new ();
   _action_manager.disable_accelerators_when_focused (entry);
   g_object_set_data (G_OBJECT(entry), "header-pane", this);
-  g_signal_connect (entry, "focus-in-event", G_CALLBACK(search_entry_focus_in_cb), NULL);
-  g_signal_connect (entry, "focus-out-event", G_CALLBACK(search_entry_focus_out_cb), NULL);
+  g_signal_connect (entry, "focus-in-event", G_CALLBACK(search_entry_focus_in_cb), nullptr);
+  g_signal_connect (entry, "focus-out-event", G_CALLBACK(search_entry_focus_out_cb), nullptr);
   g_signal_connect (entry, "activate", G_CALLBACK(search_entry_activated), this);
   entry_changed_tag = g_signal_connect (entry, "changed", G_CALLBACK(search_entry_changed), this);
 
-  gtk_entry_set_icon_from_stock( GTK_ENTRY( entry ),
-                                 GTK_ENTRY_ICON_PRIMARY,
-                                 GTK_STOCK_FIND);
-  gtk_entry_set_icon_from_stock( GTK_ENTRY( entry ),
-                                 GTK_ENTRY_ICON_SECONDARY,
-                                 GTK_STOCK_CLEAR );
+  gtk_entry_set_icon_from_icon_name ( GTK_ENTRY( entry ),
+                                      GTK_ENTRY_ICON_PRIMARY,
+                                      "edit-find");
+  gtk_entry_set_icon_from_icon_name ( GTK_ENTRY( entry ),
+                                      GTK_ENTRY_ICON_SECONDARY,
+                                      "edit-clear" );
 
   bool regex = _prefs.get_flag ("use-regex", false);
   GtkWidget * menu = gtk_menu_new ();
@@ -1910,7 +1897,7 @@ HeaderPane :: create_filter_entry ()
     mode = 1;
   else
     mode = 0;
-  GSList * l = 0;
+  GSList * l = nullptr;
   for (int i=0, qty=G_N_ELEMENTS(mode_strings); i<qty; ++i) {
     GtkWidget * w = gtk_radio_menu_item_new_with_label (l, _(mode_strings[i]));
     l = gtk_radio_menu_item_get_group (GTK_RADIO_MENU_ITEM(w));
@@ -1942,7 +1929,7 @@ gboolean
 HeaderPane :: on_selection_changed_idle (gpointer self_gpointer)
 {
   HeaderPane * self (static_cast<HeaderPane*>(self_gpointer));
-  const bool have_article = self->get_first_selected_article() != 0;
+  const bool have_article = self->get_first_selected_article() != nullptr;
   static const char* actions_that_need_an_article[] = {
     "download-selected-article",
     "save-articles",
@@ -1988,21 +1975,21 @@ HeaderPane :: HeaderPane (ActionManager       & action_manager,
   _prefs (prefs),
   _group_prefs (group_prefs),
   _wait (wait),
-  _atree (0),
-  _root (0),
-  _tree_view (0),
-  _tree_store (0),
+  _atree (nullptr),
+  _root (nullptr),
+  _tree_view (nullptr),
+  _tree_store (nullptr),
   _selection_changed_idle_tag (0),
+  _fg(prefs.get_color_str_wo_fallback ("text-color-fg")),
+  _bg(prefs.get_color_str_wo_fallback ("text-color-bg")),
   _cache (cache),
   _gui (gui),
-  _cleared (true),
-  _fg(prefs.get_color_str_wo_fallback ("text-color-fg")),
-  _bg(prefs.get_color_str_wo_fallback ("text-color-bg"))
+  _cleared (true)
 {
 
   // init the icons
   for (guint i=0; i<ICON_QTY; ++i)
-    _icons[i].pixbuf = gdk_pixbuf_new_from_inline (-1, _icons[i].pixbuf_txt, FALSE, 0);
+    _icons[i].pixbuf = load_icon (_icons[i].icon_file);
 
   // initialize the show type...
   const std::string show_type_str (prefs.get_string ("header-pane-show-matching", "articles"));
@@ -2030,11 +2017,11 @@ HeaderPane :: HeaderPane (ActionManager       & action_manager,
   /* intercept ENTER and left/right for article selection */
   g_signal_connect (w, "key-press-event", G_CALLBACK(on_keyboard_button_pressed), this);
 
-  g_signal_connect (w, "row-collapsed", G_CALLBACK(row_collapsed_cb), NULL);
-  g_signal_connect (w, "row-expanded", G_CALLBACK(row_expanded_cb), NULL);
+  g_signal_connect (w, "row-collapsed", G_CALLBACK(row_collapsed_cb), nullptr);
+  g_signal_connect (w, "row-expanded", G_CALLBACK(row_expanded_cb), nullptr);
   g_signal_connect (w, "popup-menu", G_CALLBACK(on_popup_menu), this);
   g_signal_connect (w, "row-activated", G_CALLBACK(on_row_activated), this);
-  GtkWidget * scroll = gtk_scrolled_window_new (0, 0);
+  GtkWidget * scroll = gtk_scrolled_window_new (nullptr, nullptr);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW(scroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
   gtk_container_add (GTK_CONTAINER(scroll), w);
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW(scroll), GTK_SHADOW_IN);
@@ -2110,7 +2097,7 @@ namespace
       select_subpaths (view, model, sel, path);
     }
 
-    g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
+    g_list_foreach (list, (GFunc)gtk_tree_path_free, nullptr);
     g_list_free (list);
   }
 }
@@ -2131,7 +2118,7 @@ void
 HeaderPane :: expand_selected ()
 {
   // get a list of paths
-  GtkTreeModel * model (0);
+  GtkTreeModel * model (nullptr);
   GtkTreeView * view (GTK_TREE_VIEW (_tree_view));
   GtkTreeSelection * selection (gtk_tree_view_get_selection (view));
   GList * list (gtk_tree_selection_get_selected_rows (selection, &model));
@@ -2141,7 +2128,7 @@ HeaderPane :: expand_selected ()
     gtk_tree_view_expand_row (view, path, true);
     gtk_tree_view_expand_to_path (view, path);
   }
-  g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
+  g_list_foreach (list, (GFunc)gtk_tree_path_free, nullptr);
   g_list_free (list);
 }
 
@@ -2159,20 +2146,20 @@ namespace
 
   struct TreeIteratorNext: public TreeIterFunctor {
     virtual ~TreeIteratorNext () {}
-    virtual bool operator ()(GtkTreeModel * model, GtkTreeIter * iter) const {
+    bool operator ()(GtkTreeModel * model, GtkTreeIter * iter) const override {
       return model && PAN_TREE_STORE(model)->get_next (iter);
     }
-    virtual bool front (GtkTreeModel* model, GtkTreeIter* setme) const {
+    bool front (GtkTreeModel* model, GtkTreeIter* setme) const override {
       return model && PAN_TREE_STORE(model)->front (setme);
     }
   };
 
   struct TreeIteratorPrev: public TreeIterFunctor {
     virtual ~TreeIteratorPrev () {}
-    bool operator()(GtkTreeModel * model, GtkTreeIter * iter) const {
+    bool operator()(GtkTreeModel * model, GtkTreeIter * iter) const override {
       return model && PAN_TREE_STORE(model)->get_prev (iter);
     }
-    virtual bool front (GtkTreeModel* model, GtkTreeIter* setme) const {
+    bool front (GtkTreeModel* model, GtkTreeIter* setme) const override {
       return model && PAN_TREE_STORE(model)->back (setme);
     }
   };
@@ -2184,32 +2171,38 @@ namespace
   struct ArticleExists: public ArticleTester {
     virtual ~ArticleExists() {}
     ArticleExists () {}
-    virtual bool operator()(const Article&) const { return true; }
+    bool operator()(const Article&) const override { return true; }
   };
 
   struct ArticleIsFlagged: public ArticleTester {
     virtual ~ArticleIsFlagged () {}
     const Article* article;
     ArticleIsFlagged (const Article* a) : article(a) {}
-    virtual bool operator()(const Article& a) const
+    bool operator()(const Article& a) const override
       { return a.get_flag() && a.message_id != article->message_id; }
   };
 
   struct ArticleIsParentOf: public ArticleTester {
     virtual ~ArticleIsParentOf () {}
     ArticleIsParentOf (const Data::ArticleTree& tree, const Article* a) {
-      const Article * parent = a ? tree.get_parent(a->message_id) : 0;
+      const Article * parent = a ? tree.get_parent(a->message_id) : nullptr;
       _mid = parent ? parent->message_id : "";
     }
     Quark _mid;
-    virtual bool operator()(const Article& a) const { return _mid==a.message_id; }
+    bool operator()(const Article& a) const override
+    {
+      return _mid==a.message_id;
+    }
   };
 
   struct ArticleIsUnread: public ArticleTester {
     virtual ~ArticleIsUnread () {}
     ArticleIsUnread (const Data& data): _data(data) {}
     const Data& _data;
-    virtual bool operator()(const Article& a) const { return !_data.is_read(&a); }
+    bool operator()(const Article& a) const override
+    {
+      return !_data.is_read(&a);
+    }
   };
 
   struct ArticleIsNotInThread: public ArticleTester {
@@ -2228,7 +2221,7 @@ namespace
         a = parent;
       }
     }
-    virtual bool operator()(const Article& a) const {
+    bool operator()(const Article& a) const override {
       return _root != get_root_mid(&a);
     }
   };
@@ -2238,7 +2231,7 @@ namespace
     ArticleIsUnreadAndNotInThread (const Data& data, const Data::ArticleTree& tree, const Article* a): _aiu (data), _ainit(tree, a) {}
     const ArticleIsUnread _aiu;
     const ArticleIsNotInThread _ainit;
-    virtual bool operator()(const Article& a) const {
+    bool operator()(const Article& a) const override {
       return _aiu(a) && _ainit(a);
     }
   };
@@ -2251,14 +2244,15 @@ namespace
     virtual ~SelectFunctor () {}
     SelectFunctor (GtkTreeView * view): _view(view) {}
     GtkTreeView * _view;
-    virtual void operator() (GtkTreeModel* model, GtkTreeIter* iter, const Article&) {
+void operator() (GtkTreeModel* model, GtkTreeIter* iter, const Article&) override
+    {
       GtkTreeSelection * sel (gtk_tree_view_get_selection (_view));
       gtk_tree_selection_unselect_all (sel);
       GtkTreePath * path = gtk_tree_model_get_path (model, iter);
       gtk_tree_view_expand_row (_view, path, true);
       gtk_tree_view_expand_to_path (_view, path);
-      gtk_tree_view_set_cursor (_view, path, NULL, FALSE);
-      gtk_tree_view_scroll_to_cell (_view, path, NULL, true, 0.5f, 0.0f);
+      gtk_tree_view_set_cursor (_view, path, nullptr, FALSE);
+      gtk_tree_view_scroll_to_cell (_view, path, nullptr, true, 0.5f, 0.0f);
       gtk_tree_path_free (path);
     }
   };
@@ -2267,9 +2261,10 @@ namespace
     virtual ~ReadFunctor() {}
     ReadFunctor (GtkTreeView * view, ActionManager& am): SelectFunctor(view), _am(am) {}
     ActionManager& _am;
-    virtual void operator() (GtkTreeModel* model, GtkTreeIter* iter, const Article& a) {
+    void operator() (GtkTreeModel* model, GtkTreeIter* iter, const Article& a) override
+    {
       SelectFunctor::operator() (model, iter, a);
-      maybe_activate_on_idle (_view, gtk_tree_model_get_path(model,iter), 0);
+      maybe_activate_on_idle (_view, gtk_tree_model_get_path(model,iter), nullptr);
     }
   };
 }
@@ -2286,10 +2281,10 @@ HeaderPane :: find_next_iterator_from (GtkTreeModel            * model,
                                        RowActionFunctor        & success_func,
                                        bool                      test_the_start_pos)
 {
-  g_assert (start_pos!=0);
+  g_assert (start_pos!=nullptr);
   GtkTreeIter march = *start_pos;
   bool success (false);
-  const Article *article (0);
+  const Article *article (nullptr);
   for (;;)
   {
     if (test_the_start_pos)
@@ -2313,10 +2308,10 @@ namespace
 {
   bool get_first_selection (GtkTreeSelection * sel, GtkTreeIter * setme)
   {
-    GtkTreeModel * model (0);
+    GtkTreeModel * model (nullptr);
     GList * list (gtk_tree_selection_get_selected_rows (sel, &model));
     const bool found (list && gtk_tree_model_get_iter (model, setme, (GtkTreePath*)(list->data)));
-    g_list_foreach (list, (GFunc)gtk_tree_path_free, NULL);
+    g_list_foreach (list, (GFunc)gtk_tree_path_free, nullptr);
     g_list_free (list);
     return found;
   }
@@ -2454,7 +2449,7 @@ void
 HeaderPane :: refresh_font ()
 {
   if (!_prefs.get_flag ("header-pane-font-enabled", false))
-    gtk_widget_override_font (_tree_view, 0);
+    gtk_widget_override_font (_tree_view, nullptr);
   else {
     const std::string str (_prefs.get_string ("header-pane-font", "Sans 10"));
     PangoFontDescription * pfd (pango_font_description_from_string (str.c_str()));
@@ -2482,7 +2477,7 @@ HeaderPane :: on_prefs_string_changed (const StringView& key, const StringView&)
 }
 
 void
-HeaderPane :: on_prefs_color_changed  (const StringView& key, const GdkColor&)
+HeaderPane :: on_prefs_color_changed  (const StringView& key, const GdkRGBA&)
 {
   if (key == "text-color-fg" || key == "text-color-bg")
   {
@@ -2640,7 +2635,7 @@ struct HeaderPane::SimilarWalk: public PanTreeStore::WalkFunctor
 
         retval = (double)common_chars / a.len;
       }
-        else // more than one word, so count common words
+      else // more than one word, so count common words
       {
         StringView tok;
         int str1_words(0), common_words(0);
