@@ -23,7 +23,9 @@
 #include <config.h>
 #include <signal.h>
 #include <glib/gi18n.h>
-#include "gtk-compat.h"
+#include <gtk/gtk.h>
+#include <gdk/gdk.h>
+#include "pan/gui/load-icon.h"
 #include <gmime/gmime.h>
 #include <gio/gio.h>
 
@@ -50,12 +52,10 @@ extern "C" {
 #endif
 
 #ifdef HAVE_GKR
-#if GTK_CHECK_VERSION(3,0,0)
   #define GCR_API_SUBJECT_TO_CHANGE
   #include <libsecret/secret.h>
   #include <gcr/gcr.h>
   #undef GCR_API_SUBJECT_TO_CHANGE
-#endif /* GTK_CHECK_VERSION(3,0,0) */
 #endif
 
 #include <config.h>
@@ -72,7 +72,6 @@ extern "C" {
 #include <pan/tasks/nzb.h>
 #include <pan/data-impl/data-impl.h>
 #include <pan/data-impl/data-io.h>
-#include <pan/icons/pan-pixbufs.h>
 #include "gui.h"
 #include "group-prefs.h"
 #include "prefs-file.h"
@@ -83,6 +82,7 @@ extern "C" {
 //#define DEBUG_LOCALE 1
 //#define DEBUG_PARALLEL 1
 
+
 using namespace pan;
 
 namespace
@@ -92,7 +92,7 @@ namespace
 
 namespace
 {
-  GMainLoop * nongui_gmainloop (0);
+  GMainLoop * nongui_gmainloop (nullptr);
 
   void mainloop ()
   {
@@ -119,14 +119,7 @@ namespace
 
   gboolean delete_event_cb (GtkWidget * w, GdkEvent *, gpointer user_data)
   {
-    if (static_cast<Prefs*>(user_data)->get_flag("status-icon", false))
-    {
-      gtk_widget_hide(w);
-    }
-    else
-    {
-      mainloop_quit ();
-    }
+    mainloop_quit ();
     return true; // don't invoke the default handler that destroys the widget
   }
 
@@ -193,16 +186,16 @@ namespace
   };
 
   struct Icon {
-  const guint8 * pixbuf_txt;
-  GdkPixbuf * pixbuf;
+      char const *pixbuf_file;
+      GdkPixbuf *pixbuf;
   } status_icons[NUM_STATUS_ICONS] = {
-    { icon_status_online,          0 },
-    { icon_status_offline,         0 },
-    { icon_status_active,          0 },
-    { icon_status_queue_empty,     0 },
-    { icon_status_error,           0 },
-    { icon_status_idle,            0 },
-    { icon_status_new_articles,    0 }
+    { "icon_status_online.png",          nullptr },
+    { "icon_status_offline.png",         nullptr },
+    { "icon_status_active.png",          nullptr },
+    { "icon_status_queue_empty.png",     nullptr },
+    { "icon_status_error.png",           nullptr },
+    { "icon_status_idle.png",            nullptr },
+    { "icon_status_new_articles.png",    nullptr }
   };
 
 
@@ -242,7 +235,6 @@ namespace
 
     static gboolean status_icon_periodic_refresh (gpointer p)
     {
-      static_cast<StatusIconListener*>(p)->update_status_tooltip();
       return true;
     }
 
@@ -251,7 +243,7 @@ namespace
                                char *action,
                                gpointer user_data)
     {
-      notify_notification_close (notification, NULL);
+      notify_notification_close (notification, nullptr);
 
       StatusIconListener* s = static_cast<StatusIconListener*>(user_data);
       gtk_widget_show (GTK_WIDGET(s->root));
@@ -271,20 +263,16 @@ namespace
 
     bool n() { return notif_shown; }
 
-    StatusIconListener(GtkStatusIcon * i, GtkWidget* r, Prefs& p, Queue& q, Data& d, bool v) :
-      icon(i), root(r), prefs(p), queue(q), data(d), minimized(v),
-      tasks_active(0), tasks_total(0), notif_shown(false)
+    StatusIconListener(GtkWidget* r, Prefs& p, Queue& q, Data& d, bool v) :
+      queue(q), data(d), tasks_active(0), tasks_total(0),
+      minimized(v), notif_shown(false), prefs(p), root(r)
     {
       prefs.add_listener(this);
       queue.add_listener(this);
       data.add_listener(this);
       is_online = q.is_online();
 
-      update_status_tooltip();
       status_icon_timeout_tag = g_timeout_add (500, status_icon_periodic_refresh, this);
-
-      update_status_icon(ICON_STATUS_IDLE);
-
     }
 
     ~StatusIconListener()
@@ -296,69 +284,48 @@ namespace
 #ifdef HAVE_LIBNOTIFY
       foreach(std::set<GObject*>, _notifs, it)
       {
-        notify_notification_close(NOTIFY_NOTIFICATION(*it), NULL);
+        notify_notification_close(NOTIFY_NOTIFICATION(*it), nullptr);
         g_object_unref (*it);
       }
 #endif
     }
 
     /* prefs::listener */
-    virtual void on_prefs_flag_changed (const StringView &key, bool value)
+    void on_prefs_flag_changed(StringView const &key, bool value) override
     {
-       if(key == "status-icon")
-         gtk_status_icon_set_visible(icon, value);
     }
 
-    virtual void on_prefs_int_changed (const StringView& key, int color) {}
-    virtual void on_prefs_string_changed (const StringView& key, const StringView& value) {}
-    virtual void on_prefs_color_changed (const StringView& key, const GdkColor& color) {}
-
-    void update_status_tooltip()
+    void on_prefs_int_changed(StringView const &key, int color) override
     {
-      char buf[512];
-      g_snprintf(buf,sizeof(buf), "<i>Pan %s : %s</i> \n"
-                                  "<b>Tasks running:</b> %d\n"
-                                  "<b>Total queued:</b> %d\n"
-                                  "<b>Speed:</b> %.1f KiBps\n",
-                                  PACKAGE_VERSION,
-                                  is_online ? "online" : "offline",
-                                  tasks_active,
-                                  tasks_total,
-                                  queue.get_speed_KiBps());
-
-      gtk_status_icon_set_tooltip_markup(icon, buf);
-
     }
 
-    void update_status_icon(StatusIcons si)
+    void on_prefs_string_changed(StringView const &key,
+                                 StringView const &value) override
     {
-      if (si==ICON_STATUS_IDLE)
-      {
-        if (is_online)
-          gtk_status_icon_set_from_pixbuf(icon, status_icons[ICON_STATUS_ONLINE].pixbuf);
-        else
-          gtk_status_icon_set_from_pixbuf(icon, status_icons[ICON_STATUS_OFFLINE].pixbuf);
-      } else
-        gtk_status_icon_set_from_pixbuf(icon, status_icons[si].pixbuf);
     }
 
-    void notify_of(StatusIcons si, const char* body, const char* summary)
+    void on_prefs_color_changed(StringView const &key,
+                                GdkRGBA const &color) override
+    {
+    }
+
+    void notify_of(StatusIcons si, char const *body, char const *summary)
     {
 #ifdef HAVE_LIBNOTIFY
       if (!body || !summary) return;
       if (!prefs.get_flag("use-notify", false)) return;
 
-      NotifyNotification *notif(0);
-      GError* error(0);
+      NotifyNotification *notif(nullptr);
+      GError* error(nullptr);
 
 #ifdef NOTIFY_CHECK_VERSION
   #if NOTIFY_CHECK_VERSION (0, 7, 0)
-      notif=notify_notification_new(summary, body, NULL);
+      notif=notify_notification_new(summary, body, nullptr);
   #else
-      notif=notify_notification_new(summary, body, NULL,NULL);
+      notif=notify_notification_new(summary, body, nullptr,nullptr);
   #endif
 #else
-      notif=notify_notification_new(summary, body, NULL,NULL);
+      notif=notify_notification_new(summary, body, nullptr,nullptr);
 #endif
       if (!notif) return;
 
@@ -366,7 +333,7 @@ namespace
 
       notify_notification_set_icon_from_pixbuf(notif, status_icons[si].pixbuf);
       notify_notification_set_timeout (notif,5000);
-      notify_notification_add_action(notif,"close",_("Maximize"),NOTIFY_ACTION_CALLBACK(notif_maximize_cb),this,NULL);
+      notify_notification_add_action(notif,"close",_("Maximize"),NOTIFY_ACTION_CALLBACK(notif_maximize_cb),this,nullptr);
       g_signal_connect (G_OBJECT(notif), "closed", G_CALLBACK(notif_close_cb), this);
 
       notify_notification_show (notif, &error);
@@ -381,60 +348,46 @@ namespace
     }
 
     /* queue::listener */
-    virtual void on_queue_task_active_changed (Queue&, Task&, bool active) {}
-    virtual void on_queue_tasks_added (Queue&, int index UNUSED, int count)
+    void on_queue_task_active_changed (Queue&, Task&, bool active) override {}
+    void on_queue_tasks_added (Queue&, int index UNUSED, int count) override
     {
       tasks_total += count;
-      update_status_tooltip();
-      update_status_icon(ICON_STATUS_ACTIVE);
     }
-    virtual void on_queue_task_removed (Queue&, Task&, int pos UNUSED)
+    void on_queue_task_removed (Queue&, Task&, int pos UNUSED) override
     {
-      update_status_icon(ICON_STATUS_ACTIVE);
     }
-    virtual void on_queue_task_moved (Queue&, Task&, int new_pos UNUSED, int old_pos UNUSED) {}
-    virtual void on_queue_connection_count_changed (Queue&, int count) {}
-    virtual void on_queue_size_changed (Queue&, int active, int total)
+    void on_queue_task_moved (Queue&, Task&, int new_pos UNUSED, int old_pos UNUSED) override {}
+    void on_queue_connection_count_changed (Queue&, int count) override {}
+    void on_queue_size_changed (Queue&, int active, int total) override
     {
       tasks_total = total;
       tasks_active = active;
-      if (tasks_total == 0 || tasks_active == 0)
-        update_status_icon(ICON_STATUS_IDLE);
-      update_status_tooltip();
     }
 
-    virtual void on_queue_online_changed (Queue&, bool online)
+    void on_queue_online_changed (Queue&, bool online) override
     {
       is_online = online;
-      if (tasks_total)
-        update_status_icon(ICON_STATUS_ACTIVE);
-      else
-        update_status_icon(ICON_STATUS_IDLE);
-
-      update_status_tooltip();
     }
 
-    virtual void on_queue_error (Queue&, const StringView& message)
+    void on_queue_error(Queue &, StringView const &message) override
     {
-      update_status_icon(ICON_STATUS_ERROR);
       if (n()) return;
       notif_shown = true;
       notify_of(ICON_STATUS_ERROR, message.str, _("An error has occurred!"));
     }
 
-    virtual void on_queue_size_changed (Queue&, unsigned long, unsigned long) {}
-
     /* data::listener */
-    virtual void on_group_entered (const Quark& group, unsigned long unread, unsigned long total)
+    void on_group_entered(Quark const &group,
+                          Article_Count unread,
+                          Article_Count total) override
     {
 
-      if (unread)
+      if (static_cast<uint64_t>(unread) != 0)
       {
-        update_status_icon(ICON_STATUS_NEW_ARTICLES);
         if (n()) return;
         notif_shown = true;
-        const char* summary = _("New Articles!");
-        const char* body = _("There are new\narticles available.");
+        char const *summary = _("New Articles!");
+        char const *body = _("There are new\narticles available.");
         notify_of(ICON_STATUS_NEW_ARTICLES, body, summary);
       }
     }
@@ -471,51 +424,16 @@ namespace
     d->gui.do_work_online(!d->queue.is_online());
   }
 
-  void status_icon_popup_menu (GtkStatusIcon *icon,
-                               guint button,
-                               guint activation_time,
-                               gpointer data)
-  {
-    GtkMenu * menu = GTK_MENU(data);
-    gtk_menu_popup(menu, NULL, NULL, NULL, NULL, button, activation_time);
-  }
-
-  static QueueAndGui* queue_and_gui(0);
+  static QueueAndGui* queue_and_gui(nullptr);
 
   void run_pan_with_status_icon (GtkWindow * window, GdkPixbuf * pixbuf, Queue& queue, Prefs & prefs, Data& data, GUI* _gui)
   {
 
-    GUI& gui (*_gui);
+    _status_icon = new StatusIconListener(GTK_WIDGET(window), prefs, queue, data, prefs.get_flag("start-minimized", false));
 
+    // required to show Pan icon in notification
     for (guint i=0; i<NUM_STATUS_ICONS; ++i)
-      status_icons[i].pixbuf = gdk_pixbuf_new_from_inline (-1, status_icons[i].pixbuf_txt, FALSE, 0);
-
-    GtkStatusIcon * icon = gtk_status_icon_new_from_pixbuf (status_icons[ICON_STATUS_IDLE].pixbuf);
-    GtkWidget * menu = gtk_menu_new ();
-
-    GtkWidget * menu_quit = gtk_image_menu_item_new_from_stock ( GTK_STOCK_QUIT, NULL);
-    g_signal_connect(menu_quit, "activate", G_CALLBACK(mainloop_quit), NULL);
-
-    gtk_status_icon_set_visible(icon, prefs.get_flag("status-icon", false));
-    StatusIconListener* pl = _status_icon = new StatusIconListener(icon, GTK_WIDGET(window), prefs, queue, data, prefs.get_flag("start-minimized", false));
-
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), menu_quit);
-
-    const char* label_names[] = {_("Toggle on/offline")};
-
-    GtkWidget* labels[G_N_ELEMENTS(label_names)];
-
-    labels[0] = gtk_menu_item_new_with_label(label_names[0]);
-    queue_and_gui = new QueueAndGui(queue, gui);
-    g_signal_connect(labels[0], "activate", G_CALLBACK(work_online_cb), queue_and_gui);
-
-    for (int i=0; i < G_N_ELEMENTS(label_names); ++i)
-      gtk_menu_shell_append(GTK_MENU_SHELL(menu), labels[i]);
-
-    gtk_widget_show_all(menu);
-    g_signal_connect(icon, "activate", G_CALLBACK(status_icon_activate), window);
-    g_signal_connect(icon, "popup-menu", G_CALLBACK(status_icon_popup_menu), menu);
-
+      status_icons[i].pixbuf = load_icon (status_icons[i].pixbuf_file);
   }
 
 
@@ -532,7 +450,7 @@ namespace
     const gulong delete_cb_id =  g_signal_connect (window, "delete-event", G_CALLBACK(delete_event_cb), &prefs);
 
     gtk_container_add (GTK_CONTAINER(window), gui.root());
-    const bool minimized(prefs.get_flag("start-minimized", false));
+    bool const minimized(prefs.get_flag("start-minimized", false));
 
     if (minimized)
       gtk_window_iconify (window);
@@ -577,17 +495,17 @@ namespace
     ~PanKiller() { q.remove_listener(this); }
 
     /** Method from Queue::Listener interface: quits program on zero sized Q*/
-    void on_queue_size_changed (Queue&, int active, int total)
+    void on_queue_size_changed (Queue&, int active, int total) override
       {  if (!active && !total) mainloop_quit();  }
 
     // all below methods from Queue::Listener interface are noops
-    void on_queue_task_active_changed (Queue&, Task&, bool) {}
-    void on_queue_tasks_added (Queue&, int , int ) {}
-    void on_queue_task_removed (Queue&, Task&, int) {}
-    void on_queue_task_moved (Queue&, Task&, int, int) {}
-    void on_queue_connection_count_changed (Queue&, int) {}
-    void on_queue_online_changed (Queue&, bool) {}
-    void on_queue_error (Queue&, const StringView&) {}
+    void on_queue_task_active_changed (Queue&, Task&, bool) override {}
+    void on_queue_tasks_added (Queue&, int , int ) override {}
+    void on_queue_task_removed (Queue&, Task&, int) override {}
+    void on_queue_task_moved (Queue&, Task&, int, int) override {}
+    void on_queue_connection_count_changed (Queue&, int) override {}
+    void on_queue_online_changed (Queue&, bool) override {}
+    void on_queue_error (Queue&, const StringView&) override {}
 
   private:
     Queue & q;
@@ -617,8 +535,8 @@ namespace
     cerr.ios::rdbuf( t2 );
     err.ios::rdbuf( tmp );
 
-    freopen( "CONOUT$", "w", stdout );
-    freopen( "CONOUT$", "w", stderr );
+    (void)freopen( "CONOUT$", "w", stdout );
+    (void)freopen( "CONOUT$", "w", stderr );
   }
 #else
   void console()
@@ -630,12 +548,13 @@ namespace
   void usage ()
   {
     console();
-    std::cerr << "Pan " << VERSION << "\n\n" <<
+    std::cerr << "Pan " << PAN_VERSION << "\n\n" <<
 _("General Options\n"
 "  -h, --help               Show this usage information and exit.\n"
 "  -v, --version            Print release version and exit.\n"
 "  --verbose                Be verbose (in non-GUI mode).\n"
-"  --debug                  Run in debug mode.\n"
+"  --debug                  Run in debug mode. Use --debug twice for verbose debug.\n"
+"  --debug-ssl              Run in TLS (aka SSL) debug mode.\n"
 "\n"
 "URL Options\n"
 /** NOT IMPLEMENTED
@@ -647,7 +566,7 @@ _("General Options\n"
 "                           the message-id article to standard output.\n"
 "\n"
 "NZB Batch Options\n"
-"  --nzb file1 file2 …      Process NZB files in non-GUI mode.\n"
+"  --nzb file1 file2 ...    Process NZB files in non-GUI mode.\n"
 "  -o path, --output=path   Path to save attachments listed in the NZB file(s).\n"
 "  --no-gui                 Only show console output, not the download queue.\n") << std::endl;
   }
@@ -657,9 +576,6 @@ _("General Options\n"
    ** DBUS
    ***/
 
-  #define PAN_DBUS_SERVICE_NAME      "news.pan.NZB"
-  #define PAN_DBUS_SERVICE_PATH      "/news/pan/NZB"
-
   /** Struct for dbus handling */
   struct Pan
   {
@@ -677,21 +593,20 @@ _("General Options\n"
     GDBusInterfaceVTable ifacetable;
 
     Pan(Data& d, Queue& q, ArticleCache& c, EncodeCache& ec, Prefs& p, GroupPrefs& gp) :
-      dbus_id(-1), busnodeinfo(0),
-      data(d), queue(q), cache(c), encode_cache(ec), prefs(p), group_prefs(gp),
+      data(d), queue(q),
+      cache(c), encode_cache(ec), prefs(p), group_prefs(gp), busnodeinfo(nullptr), dbus_id(-1),
       lost_name(false), name_valid(false)
       {}
   };
 
-  static void
-  nzb_method_call  (GDBusConnection      *connection,
-                   const gchar           *sender,
-                   const gchar           *object_path,
-                   const gchar           *interface_name,
-                   const gchar           *method_name,
-                   GVariant              *parameters,
-                   GDBusMethodInvocation *invocation,
-                   gpointer               user_data)
+  static void nzb_method_call(GDBusConnection *connection,
+                              gchar const *sender,
+                              gchar const *object_path,
+                              gchar const *interface_name,
+                              gchar const *method_name,
+                              GVariant *parameters,
+                              GDBusMethodInvocation *invocation,
+                              gpointer user_data)
   {
 
     Pan* pan(static_cast<Pan*>(user_data));
@@ -729,39 +644,37 @@ _("General Options\n"
       }
     }
 
-    g_dbus_method_invocation_return_value (invocation, NULL);
+    g_dbus_method_invocation_return_value (invocation, nullptr);
   }
 
-  static GDBusConnection *dbus_connection(NULL);
+  static GDBusConnection *dbus_connection(nullptr);
 
-  static const gchar xml[]=
-  "<node>"
-  "  <interface name='news.pan.NZB'>"
-  "    <method name='NZBEnqueue'>"
-  "      <arg type='s' name='groups'    direction='in'/>"
-  "      <arg type='s' name='nzb_files' direction='in'/>"
-  "      <arg type='s' name='nzb_path'  direction='in'/>"
-  "      <arg type='b' name='nzb'       direction='in'/>"
-  "    </method>"
-  "  </interface>"
-  "</node>";
+  static const gchar xml[] =
+    "<node>"
+    "  <interface name='news.pan.NZB'>"
+    "    <method name='NZBEnqueue'>"
+    "      <arg type='s' name='groups'    direction='in'/>"
+    "      <arg type='s' name='nzb_files' direction='in'/>"
+    "      <arg type='s' name='nzb_path'  direction='in'/>"
+    "      <arg type='b' name='nzb'       direction='in'/>"
+    "    </method>"
+    "  </interface>"
+    "</node>";
 
-
-  static void
-  on_bus_acquired (GDBusConnection *connection,
-                   const gchar     *name,
-                   gpointer         user_data)
+  static void on_bus_acquired(GDBusConnection *connection,
+                              gchar const *name,
+                              gpointer user_data)
   {
     Pan* pan (static_cast<Pan*>(user_data));
     g_return_if_fail (pan);
 
-    pan->busnodeinfo = g_dbus_node_info_new_for_xml (xml, NULL);
+    pan->busnodeinfo = g_dbus_node_info_new_for_xml (xml, nullptr);
 
    /* init iface table */
     GDBusInterfaceVTable tmp =
     {
       nzb_method_call ,
-      NULL, NULL
+      nullptr, nullptr
     };
     pan->ifacetable = tmp;
 
@@ -771,16 +684,15 @@ _("General Options\n"
       pan->busnodeinfo->interfaces[0],
       &pan->ifacetable,
       pan,
-      NULL,
-      NULL
+      nullptr,
+      nullptr
     );
 
   }
 
-  static void
-  on_name_acquired (GDBusConnection *connection,
-                    const gchar     *name,
-                    gpointer         user_data)
+  static void on_name_acquired(GDBusConnection *connection,
+                               gchar const *name,
+                               gpointer user_data)
   {
 
     Pan* pan(static_cast<Pan*>(user_data));
@@ -790,10 +702,9 @@ _("General Options\n"
     pan->lost_name = false;
   }
 
-  static void
-  on_name_lost (GDBusConnection *connection,
-                const gchar     *name,
-                gpointer         user_data)
+  static void on_name_lost(GDBusConnection *connection,
+                           gchar const *name,
+                           gpointer user_data)
   {
     Pan* pan(static_cast<Pan*>(user_data));
     g_return_if_fail (pan);
@@ -814,9 +725,9 @@ _("General Options\n"
         on_bus_acquired,
         on_name_acquired,
         on_name_lost,
-        pan,NULL);
+        pan,nullptr);
 
-    dbus_connection = g_bus_get_sync  (G_BUS_TYPE_SESSION , NULL, NULL);
+    dbus_connection = g_bus_get_sync  (G_BUS_TYPE_SESSION , nullptr, nullptr);
   }
 
   static void
@@ -833,7 +744,7 @@ _("General Options\n"
 
 namespace
 {
-  GUI * gui_ptr (0);
+  GUI * gui_ptr (nullptr);
 }
 
 namespace
@@ -843,15 +754,14 @@ namespace
 
     GtkWidget* r = gtk_label_new("bla");
 
-    GdkColor def_fg, def_bg;
+    GdkRGBA def_fg, def_bg;
     std::string fg_col, bg_col;
 
     // init colors of PanColors
-#if GTK_CHECK_VERSION(3,0,0)
     GdkRGBA fg_color, bg_color;
     GtkStyleContext* ctx = gtk_widget_get_style_context(r);
     if(!ctx || !gtk_style_context_lookup_color(ctx, "color", &fg_color))
-      gdk_color_parse("black", &def_fg);
+      gdk_rgba_parse(&def_fg, "black");
     else
     {
       def_fg.red = fg_color.red;
@@ -859,25 +769,17 @@ namespace
       def_fg.blue = fg_color.blue;
     }
     if(!ctx || !gtk_style_context_lookup_color(ctx, "background-color", &bg_color))
-      gdk_color_parse("white", &def_bg);
+      gdk_rgba_parse(&def_bg, "white");
     else
     {
       def_bg.red = bg_color.red;
       def_bg.green = bg_color.red;
       def_bg.blue = bg_color.blue;
     }
-#else
-    GtkStyle *style = gtk_rc_get_style(r);
-    if(!style || !gtk_style_lookup_color(style, "text_color", &def_fg))
-      gdk_color_parse("black", &def_fg);
-    if(!style || !gtk_style_lookup_color(style, "bg_color", &def_bg))
-      gdk_color_parse("white", &def_bg);
-
-#endif
 
     //todo move to pancolors
-    fg_col = GroupPrefs::color_to_string(def_fg);
-    bg_col = GroupPrefs::color_to_string(def_bg);
+    fg_col = gdk_rgba_to_string(&def_fg);
+    bg_col = gdk_rgba_to_string(&def_bg);
 
     g_object_ref_sink (r);
     gtk_widget_destroy (r);
@@ -906,11 +808,7 @@ main (int argc, char *argv[])
 #if !GLIB_CHECK_VERSION(2,32,0)
   g_thread_init (0);
 #endif
-#ifdef HAVE_GMIME_30
   g_mime_init ();
-#else
-  g_mime_init (GMIME_ENABLE_RFC2047_WORKAROUNDS);
-#endif  
 
   bool gui(true), nzb(false), verbosed(false);
   std::string url;
@@ -923,28 +821,35 @@ main (int argc, char *argv[])
 
   for (int i=1; i<argc; ++i)
   {
-    const char * tok (argv[i]);
-    if (!memcmp(tok,"headers:", 8))
-      groups = tok+8;
-    else if (!memcmp(tok,"news:", 5))
+    char const *tok(argv[i]);
+    if (! memcmp(tok, "headers:", 8))
+      groups = tok + 8;
+    else if (! memcmp(tok, "news:", 5))
       url = tok;
-    else if (!strcmp(tok,"--no-gui") || !strcmp(tok,"--nogui"))
+    else if (! strcmp(tok, "--no-gui") || ! strcmp(tok, "--nogui"))
       gui = false;
-    else if (!strcmp (tok, "--debug")) { // use --debug --debug for verbose debug
-      if (!console_active) { console_active = true; console(); }
-      if (_debug_flag) _debug_verbose_flag = true;
-      else _debug_flag = true;
-    } else if (!strcmp (tok, "--nzb"))
+    else if (! strcmp(tok, "--debug"))
+      {
+        // use --debug --debug for verbose debug
+        if (! console_active)
+        {
+          console_active = true;
+          console();
+        }
+        if (_debug_flag)
+          _debug_verbose_flag = true;
+        else
+          _debug_flag = true;
+      }
+    else if (!strcmp (tok, "--nzb"))
       nzb = true;
-
-    // undocumented, internal(!) debug flag for ssl problems (after 0.136)
     else if (!strcmp (tok, "--debug-ssl")) {
+      // undocumented, internal(!) debug flag for ssl problems (after 0.136)
       _dbg_ssl = true;
       _dbg_file.open("ssl.debug");
     }
-
     else if (!strcmp (tok, "--version") || !strcmp (tok, "-v"))
-      { std::cerr << "Pan " << VERSION << '\n'; return EXIT_SUCCESS; }
+      { std::cerr << "Pan " << PAN_VERSION << '\n'; return EXIT_SUCCESS; }
     else if (!strcmp (tok, "-o") && i<argc-1)
       nzb_output_path = argv[++i];
     else if (!memcmp (tok, "--output=", 9))
@@ -981,27 +886,27 @@ main (int argc, char *argv[])
     return EXIT_FAILURE;
   }
 
-  Log::add_info_va (_("Pan %s started"), VERSION);
+  Log::add_info_va (_("Pan %s started"), PAN_VERSION);
 
   if (gui)
   {
     // load the preferences...
-    char * filename = g_build_filename (file::get_pan_home().c_str(), "preferences.xml", NULL);
+    char * filename = g_build_filename (file::get_pan_home().c_str(), "preferences.xml", nullptr);
     PrefsFile prefs (filename); // dummy is used to get fg/bg colors from UI
     g_free (filename);
-    filename = g_build_filename (file::get_pan_home().c_str(), "group-preferences.xml", NULL);
+    filename = g_build_filename (file::get_pan_home().c_str(), "group-preferences.xml", nullptr);
     GroupPrefs group_prefs (filename);
     g_free (filename);
 
     // instantiate the backend...
-    const int cache_megs = prefs.get_int ("cache-size-megs", 10);
+    int const cache_megs = prefs.get_int("cache-size-megs", 10);
     DataImpl data (prefs.get_string("cache-file-extension","msg"), prefs, false, cache_megs);
     ArticleCache& cache (data.get_cache ());
     EncodeCache& encode_cache (data.get_encode_cache());
     CertStore& certstore (data.get_certstore());
 
     if (nzb && data.get_servers().empty()) {
-      std::cerr << _("Please configure Pan’s news servers before using it as an nzb client.") << std::endl;
+      std::cerr << _("Please configure Pan's news servers before using it as an nzb client.") << std::endl;
        return EXIT_FAILURE;
     }
     data.set_newsrc_autosave_timeout( prefs.get_int("newsrc-autosave-timeout-min", 10 ));
@@ -1019,10 +924,12 @@ main (int argc, char *argv[])
 
       pan_dbus_init(&pan);
 
-      GError* error(NULL);
+      GError* error(nullptr);
       GVariant* var;
 
       if (!dbus_connection) goto _fail;
+
+      if (nzb)
       {
         g_dbus_connection_call_sync (dbus_connection,
                                PAN_DBUS_SERVICE_NAME,
@@ -1031,10 +938,10 @@ main (int argc, char *argv[])
                                "NZBEnqueue",
                                g_variant_new ("(sssb)",
                                   groups.c_str(), nzb_output_path.c_str(), nzb_str.c_str(),  gui, nzb),
-                               NULL,
+                               nullptr,
                                G_DBUS_CALL_FLAGS_NONE,
                                -1,
-                               NULL,
+                               nullptr,
                                &error);
 
         if (!error)
@@ -1069,7 +976,7 @@ main (int argc, char *argv[])
         // if no save path was specified, either prompt for one or
         // use the user's home directory as a fallback.
         if (nzb_output_path.empty() && gui)
-          nzb_output_path = GUI::prompt_user_for_save_path (NULL, prefs);
+          nzb_output_path = GUI::prompt_user_for_save_path (nullptr, prefs);
         if (nzb_output_path.empty()) // user pressed `cancel' when prompted
           return EXIT_FAILURE;
 
@@ -1081,20 +988,20 @@ main (int argc, char *argv[])
       }
 
       // if in non-gui mode, contains a PanKiller ptr to quit pan on queue empty
-      std::auto_ptr<PanKiller> killer;
+      std::unique_ptr<PanKiller> killer;
 
       // don't open the full-blown Pan, just act as a nzb client,
       // with a gui or without.
       if (gui) {
         TaskPane * pane = new TaskPane (queue, prefs);
         GtkWidget * w (pane->root());
-        GdkPixbuf * pixbuf = gdk_pixbuf_new_from_inline (-1, icon_pan, FALSE, 0);
+        GdkPixbuf * pixbuf = load_icon("icon_pan.png");
         gtk_window_set_default_icon (pixbuf);
         gtk_widget_show_all (w);
-        g_signal_connect (w, "destroy", G_CALLBACK(destroy_cb), 0);
-        g_signal_connect (G_OBJECT(w), "delete-event", G_CALLBACK(delete_event_cb), 0);
+        g_signal_connect (w, "destroy", G_CALLBACK(destroy_cb), nullptr);
+        g_signal_connect (G_OBJECT(w), "delete-event", G_CALLBACK(delete_event_cb), nullptr);
       } else {
-        nongui_gmainloop = g_main_loop_new (NULL, false);
+        nongui_gmainloop = g_main_loop_new (nullptr, false);
         // create a PanKiller object -- which quits pan when the queue is done
         killer.reset(new PanKiller(queue));
       }
@@ -1103,7 +1010,7 @@ main (int argc, char *argv[])
     }
     else
     {
-      GdkPixbuf * pixbuf = gdk_pixbuf_new_from_inline (-1, icon_pan, FALSE, 0);
+      GdkPixbuf* pixbuf = load_icon("icon_pan.png");
       GtkWidget * window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
       if (prefs.get_flag ("main-window-is-maximized", false))
         gtk_window_maximize (GTK_WINDOW (window));
@@ -1160,11 +1067,7 @@ main (int argc, char *argv[])
       Data::Server* s(data.find_server(*it));
       if (s && s->gkr_pw)
       {
-#if GTK_CHECK_VERSION(3,0,0)
         gcr_secure_memory_free(s->gkr_pw);
-#else
-        gnome_keyring_memory_free(s->gkr_pw);
-#endif /* GTK_CHECK_VERSION(3,0,0) */
       }
     }
   }
