@@ -21,14 +21,13 @@
 #include <cassert>
 #include <cerrno>
 
-extern "C"
-{
-#define PROTOTYPES
+extern "C" {
 #include <stdio.h>
+}
+#define PROTOTYPES
 #include <uulib/uudeview.h>
 #include <glib/gi18n.h>
 #include <gmime/gmime-utils.h>
-}
 
 #include <fstream>
 #include <iostream>
@@ -80,27 +79,32 @@ namespace
   {
     char buf[1024];
     if (mode == TaskXOver::ALL)
-      snprintf(buf, sizeof(buf), _("Getting all headers for “%s”"),
+      snprintf(buf, sizeof(buf), _("Getting all headers for \"%s\""),
           group.c_str());
     else if (mode == TaskXOver::NEW)
-      snprintf(buf, sizeof(buf), _("Getting new headers for “%s”"),
+      snprintf(buf, sizeof(buf), _("Getting new headers for \"%s\""),
           group.c_str());
     else
       // SAMPLE
-      snprintf(buf, sizeof(buf), _("Sampling headers for “%s”"),
+      snprintf(buf, sizeof(buf), _("Sampling headers for \"%s\""),
           group.c_str());
     return std::string(buf);
   }
 }
 
-TaskXOver::TaskXOver(Data & data, const Quark & group, Mode mode,
-    unsigned long sample_size) :
-    Task("XOVER", get_description(group, mode)), _data(data), _group(group), _short_group_name(
-        get_short_name(StringView(group.c_str()))), _mode(mode), _sample_size(
-        sample_size), _days_cutoff(
-        mode == DAYS ? (time(0) - (sample_size * 24 * 60 * 60)) : 0), _group_xover_is_reffed(
-        false), _bytes_so_far(0), _parts_so_far(0ul), _articles_so_far(0ul), _total_minitasks(
-        0)
+TaskXOver::TaskXOver(Data & data, const Quark & group, Mode mode, unsigned long sample_size) :
+  Task("XOVER", get_description(group, mode)),
+  _data(data),
+  _group(group),
+  _short_group_name(get_short_name(StringView(group.c_str()))),
+  _mode(mode),
+  _sample_size(sample_size),
+  _days_cutoff(mode == DAYS ? (time(nullptr) - (sample_size * 24 * 60 * 60)) : 0),
+  _group_xover_is_reffed(false),
+  _bytes_so_far(0),
+  _parts_so_far(0ul),
+  _articles_so_far(0ul),
+  _total_minitasks(0)
 {
 
   debug("ctor for " << group);
@@ -128,10 +132,13 @@ TaskXOver::~TaskXOver()
 {
   if (_group_xover_is_reffed)
     {
-      foreach (server_to_high_t, _high, it)_data.set_xover_high (_group, it->first, it->second);
+      foreach (server_to_high_t, _high, it)
+        {
+          _data.set_xover_high (_group, it->first, it->second);
+        }
       _data.xover_unref (_group);
     }
-  _data.fire_group_entered(_group, 1, 0);
+  _data.fire_group_entered(_group, static_cast<Article_Count>(1), static_cast<Article_Count>(0));
 }
 
 void
@@ -189,8 +196,8 @@ TaskXOver::use_nntp(NNTP* nntp)
  ***/
 
 void
-TaskXOver::on_nntp_group(NNTP * nntp, const Quark & group, unsigned long qty,
-    uint64_t low, uint64_t high)
+TaskXOver::on_nntp_group(NNTP * nntp, const Quark & group, Article_Count qty,
+    Article_Number low, Article_Number high)
 {
   const Quark& servername(nntp->_server);
   CompressionType comp;
@@ -206,7 +213,7 @@ TaskXOver::on_nntp_group(NNTP * nntp, const Quark & group, unsigned long qty,
   debug(
       "got GROUP result from " << nntp->_server << " (" << nntp << "): " << " qty " << qty << " low " << low << " high " << high);
 
-  uint64_t l(low), h(high);
+  Article_Number l(low), h(high);
   _data.set_xover_low(group, nntp->_server, low);
   //std::cerr << LINE_ID << " This group's range is [" << low << "..." << high << ']' << std::endl;
 
@@ -214,13 +221,13 @@ TaskXOver::on_nntp_group(NNTP * nntp, const Quark & group, unsigned long qty,
     l = low;
   else if (_mode == SAMPLE)
     {
-      _sample_size = std::min(_sample_size, high - low);
+      _sample_size = std::min(_sample_size, static_cast<uint64_t>(high - low));
       //std::cerr << LINE_ID << " and I want to sample " <<  _sample_size << " messages..." << std::endl;
       l = std::max(low, high + 1 - _sample_size);
     }
   else
     { // NEW
-      uint64_t xh(_data.get_xover_high(group, nntp->_server));
+      Article_Number xh(_data.get_xover_high(group, nntp->_server));
       //std::cerr << LINE_ID << " current xover high is " << xh << std::endl;
       l = std::max(xh + 1, low);
     }
@@ -228,17 +235,52 @@ TaskXOver::on_nntp_group(NNTP * nntp, const Quark & group, unsigned long qty,
   if (l <= high)
     {
       //std::cerr << LINE_ID << " okay, I'll try to get articles in [" << l << "..." << h << ']' << std::endl;
-      add_steps(h - l);
+      add_steps(static_cast<uint64_t>(h - l));
       const int INCREMENT(compression_enabled ? 10000 : 1000);
       MiniTasks_t& minitasks(_server_to_minitasks[servername]);
-      for (uint64_t m = l; m <= h; m += INCREMENT)
+      //Unfortunately we need to push everything to the front of the list, so
+      //that we process all the xovers before we process the next group.
+      //But we want to fetch all the articles in order so if someone exits,
+      //on resumption we can resume from where we left off. Therefore, we build
+      //a list of things to do in reverse order
+      std::vector<MiniTask> tasks;
+      if (_mode != DAYS)
+      {
+        tasks.reserve(static_cast<uint64_t>(h - l));
+      }
+      for (Article_Number m = l; m <= h; m += INCREMENT)
         {
-          const MiniTask mt(MiniTask::XOVER, m, m + INCREMENT);
+          //A note: It may not be necessary to cap the high here, the spec isn't
+          //terribly clear about what happens if new articles come into
+          //existence on the server while it is working out the response to the
+          //xover. So be safe.
+          const MiniTask mt(MiniTask::XOVER, m, std::min(h, m + INCREMENT));
           debug(
               "adding MiniTask for " << servername << ": xover [" << mt._low << '-' << mt._high << "]");
+          if (_mode == DAYS)
+          {
+            minitasks.push_front(mt);
+            ++_total_minitasks;
+          }
+          else
+          {
+            tasks.insert(tasks.begin(), mt);
+          }
+        }
+
+      //And this reverses them again, so we're back in the right order. We don't
+      //do it for days as there's a cutoff in the receiving code that stops us
+      //as soon as we get older than the specified number of days. However, for
+      //fetching anything else, we go forward so that it's possible to carry on
+      //fetching new articles without leaving a gap after exit or crash.
+      if (_mode != DAYS)
+      {
+        for (auto const & mt : tasks)
+        {
           minitasks.push_front(mt);
           ++_total_minitasks;
         }
+      }
     }
   else
     {
@@ -257,22 +299,7 @@ namespace
     if (!view.empty())
       {
         errno = 0;
-        ul = strtoul(view.str, 0, 10);
-        if (errno)
-          ul = 0ul;
-      }
-
-    return ul;
-  }
-  uint64_t
-  view_to_ull(const StringView& view)
-  {
-    uint64_t ul = 0ul;
-
-    if (!view.empty())
-      {
-        errno = 0;
-        ul = g_ascii_strtoull(view.str, 0, 10);
+        ul = strtoul(view.str, nullptr, 10);
         if (errno)
           ul = 0ul;
       }
@@ -284,7 +311,7 @@ namespace
   header_is_nonencoded_utf8(const StringView& in)
   {
     const bool is_nonencoded(!in.strstr("=?"));
-    const bool is_utf8(g_utf8_validate(in.str, in.len, 0));
+    const bool is_utf8(g_utf8_validate(in.str, in.len, nullptr));
     return is_nonencoded && is_utf8;
   }
 }
@@ -314,7 +341,7 @@ void
 TaskXOver::on_nntp_line_process(NNTP * nntp, const StringView & line)
 {
 
-  pan_return_if_fail(nntp != 0);
+  pan_return_if_fail(nntp != nullptr);
   pan_return_if_fail(!nntp->_server.empty());
   pan_return_if_fail(!nntp->_group.empty());
 
@@ -322,13 +349,13 @@ TaskXOver::on_nntp_line_process(NNTP * nntp, const StringView & line)
 
   unsigned int lines = 0u;
   unsigned long bytes = 0ul;
-  uint64_t number = 0;
+  Article_Number number(0);
   StringView subj, author, date, mid, tmp, xref, l(line);
   std::string ref;
   bool ok = !l.empty();
   ok = ok && l.pop_token(tmp, '\t');
   if (ok)
-    number = view_to_ull(tmp);
+    number = Article_Number(tmp);
   tmp.clear();
   ok = ok && l.pop_token(subj, '\t');
   if (ok)
@@ -372,40 +399,36 @@ TaskXOver::on_nntp_line_process(NNTP * nntp, const StringView & line)
 
   if (xref.len > 6 && !strncmp(xref.str, "Xref: ", 6))
     {
-      xref = xref.substr(xref.str + 6, 0);
+      xref = xref.substr(xref.str + 6, nullptr);
       xref.trim();
     }
 
   // is this header corrupt?
-  if (!number // missing number
-  || subj.empty() // missing subject
+  if (static_cast<uint64_t>(number) == 0 // missing number
+      || subj.empty() // missing subject
       || author.empty() // missing author
       || date.empty() // missing date
       || mid.empty() // missing mid
-			|| mid.front() != '<') // corrupt mid
+      || mid.front() != '<') // corrupt mid
 	/// Concerning bug : https://bugzilla.gnome.org/show_bug.cgi?id=650042
 	/// Even if we didn't get a proper reference here, continue.
 	//|| (!ref.empty() && ref.front()!='<'))
 		return;
 
 	// if news server doesn't provide an xref, fake one
-	char * buf(0);
+	char * buf(nullptr);
 	if (xref.empty())
 		xref = buf = g_strdup_printf("%s %s:%" G_GUINT64_FORMAT,
-				nntp->_server.c_str(), nntp->_group.c_str(), number);
+				nntp->_server.c_str(), nntp->_group.c_str(), static_cast<uint64_t>(number));
 
-	uint64_t& h(_high[nntp->_server]);
+	Article_Number& h(_high[nntp->_server]);
 	h = std::max(h, number);
 
 	const char * fallback_charset = NULL; // FIXME
 
 	// are we done?
-#ifdef HAVE_GMIME_30
 	GDateTime * time_posted_gd = g_mime_utils_header_decode_date(date.str);
 	const time_t time_posted = g_date_time_to_unix(time_posted_gd);
-#else	
-	const time_t time_posted = g_mime_utils_header_decode_date(date.str, NULL);
-#endif	
 	if (_mode == DAYS && time_posted < _days_cutoff) {
 		_server_to_minitasks[nntp->_server].clear();
 		return;
@@ -424,8 +447,8 @@ TaskXOver::on_nntp_line_process(NNTP * nntp, const StringView & line)
 		++_articles_so_far;
 
 	// emit a status update
-	uint64_t& prev = _last_xover_number[nntp];
-	increment_step(number - prev);
+	Article_Number& prev = _last_xover_number[nntp];
+	increment_step(static_cast<uint64_t>(number) - static_cast<uint64_t>(prev));
 	prev = number;
 	if (!(_parts_so_far % 500))
 		set_status_va(_("%s (%lu parts, %lu articles)"),
@@ -447,7 +470,7 @@ TaskXOver::on_nntp_done(NNTP * nntp, Health health, const StringView & response)
 
   if (compression_enabled && atoi(response.str) == 0)
   {
-    std::stringstream* buffer(0);
+    std::stringstream* buffer(nullptr);
     std::stringstream out, out2;
     if (comp == HEADER_COMPRESS_XZVER || comp == HEADER_COMPRESS_DIABLO)
     {
@@ -502,7 +525,7 @@ TaskXOver::update_work(bool subtract_one_from_nntp_count)
       _state.set_completed();
       set_finished(OK);
       char str[4096];
-      g_snprintf(str, sizeof(str), _("Getting new headers for “%s” done."), _group.c_str());
+      g_snprintf(str, sizeof(str), _("Getting new headers for \"%s\" done."), _group.c_str());
       verbose (str);
     }
 }
